/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry;

import java.util.Enumeration;

import net.rim.device.api.ui.UiApplication;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.ProxyController;
import com.funambol.blackberry.controller.LoginManager;
import com.funambol.blackberry.helper.EventHelper;
import com.funambol.blackberry.source.pim.BBPIMSyncSource;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.blackberry.source.media.picture.TwoPhasesPictureSyncSource;
import com.funambol.blackberry.ui.rim.RimControllerDataFactory;
import com.funambol.blackberry.ui.rim.RimDisplayManager;
import com.funambol.blackberry.ui.SplashScreen;
import com.funambol.blackberry.ui.SplashScreenHandler;

import com.funambol.util.BlackberryUtils;

import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Appender;
import com.funambol.util.FileAppender;
import com.funambol.util.ConsoleAppender;
import com.funambol.util.MemoryAppender;
import com.funambol.util.Log;
import com.funambol.util.MultipleAppender;
import com.funambol.util.BlackberryEventLogAppender;
import com.funambol.util.ConnectionManager;
import com.funambol.syncml.spds.MappingStoreBuilder;
import com.funambol.syncml.spds.MappingManager;
import com.funambol.storage.StringKeyValueStore;
import com.funambol.storage.StringKeyValueRMSStore;
import com.funambol.client.source.AppSyncSource;

/**
 * Provides the user interface with the configuration settings, mail
 * synchronization capabilities (polling included), and contacts and calendar
 * events synchronization.
 */
public class SyncClient extends UiApplication {

    private static final String TAG_LOG = "SyncClient";
    
    private Controller      controller     = null;

    private static Appender logAppender    = null;
    private MemoryAppender  memoryAppender = null;
    private BBCustomization customization = BBCustomization.getInstance();
    private SplashScreenHandler splashScreenHandler = null;

    /**
     * The default constructor.
     */
    public SyncClient() {
    }

    /**
     * Start the sync client application, but setting up everything which is
     * required.
     */
    public void start() throws Exception {

        // First of all we initialize the log
        initLog();

        // Force the log level to a given value if specified by the customization
        if(customization.lockLogLevel()){
            Log.lockLogLevel(customization.getLockedLogLevel());
        }

        /*
         * get configurations contents from the store.
         */
        BBConfiguration configuration = BBConfiguration.getInstance();
        configuration.init(this.getClass());
        configuration.load();
        if (!configuration.isInitialized()) {
            throw new Exception(); // TODO
        }
        // The entire boot sequence is performed at log level DEBUG, then we
        // revert to the user settings
        configuration.setTempLogLevel(Log.DEBUG);

        // Create syncsource configs
        initializeSources();

        //Set the Connection parameters if the customization file has one fixed 
        //apn  
        ConnectionManager.getInstance().setFixedApn(customization.getFixedApn());

        // Initialize the MappingManagerStoreManager builder for this app
        // We must initialize the mapping store builder so that the
        // MappingManager will use a persistable store (in RMS)
        // At the moment we use an RMS store also for the BB because the 64K
        // limitation is not a problem for mappings
        MappingStoreBuilder mappingStoreBuilder = new BBSyncClientMappingStoreBuilder();
        MappingManager.setStoreBuilder(mappingStoreBuilder);

        // Create the global controller. This can be done only after the
        // AppSyncSources are created
        controller = new Controller(this, new RimControllerDataFactory());

        // Initialize the ProxyController
        configuration.setController(ProxyController.getInstance());

        // The controller must be registered in each source
        Enumeration sources = BBAppSyncSourceManager.getInstance().getRegisteredSources();
        while(sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();

            // We expect all sources to be BBAppSyncSource, but we check anyway
            if (appSource instanceof BBAppSyncSource) {
                BBAppSyncSource bbAppSyncSource = (BBAppSyncSource)appSource;
                bbAppSyncSource.setController(controller);
            }


            // TODO FIXME, this must change. It is only necessary to stop the
            // sync. Once the SyncManager will be stoppable the sync source
            // will not need the controller any longer
            SyncSource source = appSource.getSyncSource();
            if (source != null) {
                if(source instanceof BBPIMSyncSource) {
                    BBPIMSyncSource bbPIMSyncSource = (BBPIMSyncSource)source;
                    bbPIMSyncSource.setController(controller.getHomeScreenController());
                } else if(source instanceof TwoPhasesPictureSyncSource) {
                    TwoPhasesPictureSyncSource twoPhasesPictureSyncSource = (TwoPhasesPictureSyncSource)source;
                    twoPhasesPictureSyncSource.setController(controller.getHomeScreenController());
                }
            }
        }

        // Restore the user defined log level
        configuration.restoreLogLevel();
        controller.reapplyConfiguration();

        /*
         * The UI engine must be accessed holding the event lock object shared
         * with the application's main thread, otherwise an
         * IllegalStateException is thrown (the event lock is held here where
         * the screen is pushed in the foreground)
         */
        Log.debug(TAG_LOG, "eventLock");
        synchronized (getEventLock()) {
            splashScreenHandler = new SplashScreenHandler(1000, controller, this);
            splashScreenHandler.show();
        }
        Log.debug(TAG_LOG, "End");
    }

    public void activate() {
        // The application has been brought to foreground
        super.activate();

        // If we have a splash screen and the application is manually executed,
        // then the onExposed of the SplashScreen is not executed. We need to
        // manually inform the screen that it is in foreground now
        if (splashScreenHandler != null) {
            SplashScreen splashScreen = splashScreenHandler.getSplashScreen();
            if (splashScreen != null) {
                if (getActiveScreen() == splashScreen) {
                    splashScreen.onExposed();
                }
            }
        }
    }

    public void showMainScreen() {

        BBConfiguration configuration = BBConfiguration.getInstance();
        synchronized (getEventLock()) {

            LoginManager lm = new LoginManager(controller);
            lm.login();
            // The configuration may have some post startup
            // notification/operation to complete. In this case we execute the
            // provided object
            Runnable postConfigurationTask = configuration.getPostConfigurationTask();
            if (postConfigurationTask != null) {
                Thread t = new Thread(postConfigurationTask);
                t.start();
            }
        }
    }

    protected void initLog() {
        if (customization.enableFileLogging()) {
            String logFilePath = customization.getLogFileDirectory();
            String logFileName = customization.getLogFileName();
            Appender fileAppender = new FileAppender(logFilePath, logFileName);

            Appender appender;

            if (BlackberryUtils.isSimulator()) {
                // On the simulator we use the console appender and the file
                // appender so it is very convient to get the logs
                MultipleAppender appenders = new MultipleAppender();
                ConsoleAppender consoleAppender = new ConsoleAppender();
                appenders.addAppender(consoleAppender);
                appenders.addAppender(fileAppender);
                appender = appenders;
            } else {
                // On real devices we log to file only
                appender = fileAppender;
            }
            logAppender = appender;
            Log.initLog(appender, Log.TRACE);
        } else {
            MultipleAppender appenders = new MultipleAppender();
            String appName = customization.getLogAppTitle();
            Appender eventAppender = new BlackberryEventLogAppender(appName,
                    BBVersion.VERSION);
            eventAppender.initLogFile();
            appenders.addAppender(eventAppender);
            memoryAppender = new MemoryAppender();
            appenders.addAppender(memoryAppender);
            logAppender = appenders;
            Log.initLog(logAppender, Log.TRACE);
            //for customer who wants to have log locked on specified level
            if(customization.lockLogLevel()){
                Log.lockLogLevel(customization.getLockedLogLevel());
            }
        }
    }

    private void initializeSources() {

        // First of all initialize all the sources that are displayable
        int sources[] = customization.getSourcesOrder();
        int uiOrder = 0;
        for (int i=0;i<sources.length;++i) {
            int sourceId = sources[i];
            switch (sourceId) {
                case BBAppSyncSourceManager.SYNC_FLAG_CONTACTS:
                    uiOrder = initContactsSource(uiOrder);
                    break;
                case BBAppSyncSourceManager.SYNC_FLAG_CALENDAR:
                    uiOrder = initCalendarSource(uiOrder);
                    break;
                case BBAppSyncSourceManager.SYNC_FLAG_TASKS:
                    uiOrder = initTasksSource(uiOrder);
                    break;
                case BBAppSyncSourceManager.SYNC_FLAG_NOTES:
                    uiOrder = initNotesSource(uiOrder);
                    break;
                case BBAppSyncSourceManager.SYNC_FLAG_PHOTOS:
                    uiOrder = initPicturesSource(uiOrder);
                    break;
                case BBAppSyncSourceManager.SYNC_FLAG_VIDEOS:
                    uiOrder = initVideosSource(uiOrder);
                    break;
                default:
                    Log.error(TAG_LOG, "Unknown source id: " + sourceId);
            }
        }
        // Now initialize other sources (non displayable)
        initConfigSource();
    }

    private int initContactsSource(int uiOrder) {

        /** Contacts */
        if (customization.isSourceActive(BBAppSyncSourceManager.SYNC_FLAG_CONTACTS)) {
            try {
                BBAppSyncSourceManager.getInstance().setUpContactsSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error(TAG_LOG, "Could not create contacts sync source, disabling contacts", e);
                BBAppSyncSourceManager.getInstance().disableSource(
                        BBAppSyncSourceManager.SYNC_FLAG_CONTACTS);
            }
        }
        return uiOrder;
    }

    private int initCalendarSource(int uiOrder) {

        /** Calendar */
        if (customization.isSourceActive(BBAppSyncSourceManager.SYNC_FLAG_CALENDAR)) {
            try {
                BBAppSyncSourceManager.getInstance().setUpCalendarSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error(TAG_LOG, "Could not create calendar sync source, disabling calendar", e);
                BBAppSyncSourceManager.getInstance().disableSource(
                        BBAppSyncSourceManager.SYNC_FLAG_CALENDAR);
            }
        }
        return uiOrder;
    }

    private int initTasksSource(int uiOrder) {

        /** Tasks */
        if (customization.isSourceActive(BBAppSyncSourceManager.SYNC_FLAG_TASKS)) {
            try {
                BBAppSyncSourceManager.getInstance().setUpTasksSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error(TAG_LOG, "Could not create tasks sync source, disabling tasks", e);
                BBAppSyncSourceManager.getInstance().disableSource(
                        BBAppSyncSourceManager.SYNC_FLAG_TASKS);
            }
        }
        return uiOrder;
    }

    private int initNotesSource(int uiOrder) {

        /** Notes */
        if (customization.isSourceActive(BBAppSyncSourceManager.SYNC_FLAG_NOTES)) {
            try {
                BBAppSyncSourceManager.getInstance().setUpNotesSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error(TAG_LOG, "Could not create notes sync source, disabling notes", e);
                BBAppSyncSourceManager.getInstance().disableSource(
                        BBAppSyncSourceManager.SYNC_FLAG_NOTES);
            }
        }
        return uiOrder;
    }

    private int initPicturesSource(int uiOrder) {

        /** Pictures  */
        if (customization.isSourceActive(BBAppSyncSourceManager.SYNC_FLAG_PHOTOS)) {
            try {
                BBAppSyncSourceManager.getInstance().setUpPicturesSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error(TAG_LOG, "Coud not create photos sync source, disabling photos", e);
                BBAppSyncSourceManager.getInstance().disableSource(
                        BBAppSyncSourceManager.SYNC_FLAG_PHOTOS);
            }
        }
        return uiOrder;
    }

    private int initVideosSource(int uiOrder) {

        /** Videos */
        if (customization.isSourceActive(BBAppSyncSourceManager.SYNC_FLAG_VIDEOS)) {
            try {
                BBAppSyncSourceManager.getInstance().setUpVideosSource(uiOrder);
                uiOrder++;
            } catch (Throwable e) {
                Log.error(TAG_LOG, "Coud not create videos sync source, " +
                        "disabling videos", e);
                BBAppSyncSourceManager.getInstance().disableSource(
                        BBAppSyncSourceManager.SYNC_FLAG_VIDEOS);
            }
        }
        return uiOrder;
    }


    private void initConfigSource() {

        /** Config */
        try {
            BBAppSyncSourceManager.getInstance().setUpConfigSource();
        } catch (Throwable e) {
            Log.error(TAG_LOG, "Coud not create config sync source, disabling config", e);
            BBAppSyncSourceManager.getInstance().disableSource(
                    BBAppSyncSourceManager.SYNC_FLAG_CONFIG);
        }
    }

    public SourceConfig getSourceConfig(int type) {
        AppSyncSource appSource = BBAppSyncSourceManager.getInstance().getSource(type);
        SyncSource source = appSource.getSyncSource();
        if (source != null) {
            return source.getConfig();
        }
        return null;
    }

    protected SourceConfig setSourceConfig(int type, SourceConfig config) {
        AppSyncSource appSource = BBAppSyncSourceManager.getInstance().getSource(type);
        SyncSource source = appSource.getSyncSource();
        if (source != null) {
            source.setConfig(config);
        } else {
            Log.error(TAG_LOG, "Unable to set source config for type " + type);
        }
        return null;
    }

    public void setEnableBlockInvites(boolean enabled) {
        AppSyncSource appSource = BBAppSyncSourceManager.getInstance()
                .getSource(BBAppSyncSourceManager.SYNC_FLAG_CALENDAR);
        if (appSource != null) {
            BBPIMSyncSource ss = (BBPIMSyncSource) appSource.getSyncSource();
            if (ss != null) {
                ((EventHelper) ss.getHelper()).setIncomingMailListenerStatus(enabled);
            }
        }
    }

    public String getLogContent() {
        // Just for safety we check if the memoryAppender is initialized.
        // It it is not, then we have a logical error, but at least we avoid a
        // NPE
        if (memoryAppender != null) {
            return memoryAppender.getLogData();
        } else {
            return "";
        }
    }

    public static Appender getAppender() {
        return logAppender;
    }

    private class BBSyncClientMappingStoreBuilder extends MappingStoreBuilder {

        public BBSyncClientMappingStoreBuilder() {
        }

        public StringKeyValueStore createNewStore(String name) {
            return new StringKeyValueRMSStore(name);
        }
    }
}
