/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Enumeration;
import java.util.Vector;

import net.rim.device.api.i18n.Locale;
import net.rim.device.api.system.Branding;
import net.rim.device.api.system.DeviceInfo;

import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBAppSyncSourceManager;
import com.funambol.blackberry.BBLocalization;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.blackberry.source.pim.calendar.CalendarAppSyncSourceConfig;
import com.funambol.blackberry.ui.AboutScreen;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.client.configuration.Configuration;
import com.funambol.client.controller.AccountScreenController;
import com.funambol.util.HttpDigestAuthentication;
import com.funambol.syncml.spds.SourceConfig;
import com.funambol.syncml.protocol.DevInf;
import com.funambol.syncml.protocol.Ext;
import com.funambol.syncml.protocol.DataStore;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.client.ui.ConfigurationScreen;
import com.funambol.client.controller.SyncSettingsScreenController;
import com.funambol.client.controller.AdvancedSettingsScreenController;
import com.funambol.client.controller.SignupScreenController;
import com.funambol.client.source.AppSyncSourceManager;
import com.funambol.client.ui.AccountScreen;
import com.funambol.client.ui.LoginScreen;
import com.funambol.client.ui.SignupScreen;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.ErrorLog;
import com.funambol.util.HttpTransportAgent;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;

public class Controller {

    private static final String TAG_LOG = "Controller";

    private final SyncClient     client;

    private final DisplayManager displayManager;

    private ScreenDisplayController          screenDisplayController;
    private HomeScreenController             homeScreenController;
    private DialogController                 dialogController;
    
    private AccountScreenController          loginScreenController;
    private AccountScreenController          accountScreenController;
    private SignupScreenController           signupScreenController;
    private SyncSettingsScreenController     syncSettingsScreenController;
    private AdvancedSettingsScreenController advancedSettingsScreenController;

    private final ErrorLog            errors;

    private BBCustomization           customization = BBCustomization.getInstance();
    private BBLocalization            localization  = BBLocalization.getInstance();
    private BBConfiguration           configuration = BBConfiguration.getInstance();

    private BBAppSyncSourceManager    appSyncSourceManager = BBAppSyncSourceManager.getInstance();


    // Constructor
    public Controller(SyncClient client, ControllerDataFactory fact) {

        fact.setController(this);

        // This is the earliest moment at which we can initialize the
        // ProxyController
        ProxyController.getInstance(this);

        errors = new ErrorLog();

        this.displayManager = fact.getDisplayManager();

        HomeScreen homeScreen            = fact.getHomeScreen();
        ConfigurationScreen configScreen = fact.getConfigurationScreen();
        AboutScreen aboutScreen          = fact.getAboutScreen();
        LoginScreen loginScreen          = fact.getLoginScreen();
        SignupScreen signupScreen        = null;
        if(customization.getMobileSignupEnabled()) {
            signupScreen = fact.getSignupScreen();
        }

        this.client = client;

        screenDisplayController = new ScreenDisplayController(displayManager, homeScreen,
                configScreen, aboutScreen, loginScreen, signupScreen, null);
        homeScreenController = new HomeScreenController(this, homeScreen, client);
        dialogController = new DialogController(displayManager);
        homeScreen.initialize(homeScreenController);
    }

    /**
     * @return the ScreenDisplayController
     */
    public ScreenDisplayController getScreenDisplayController() {
        return screenDisplayController;
    }

    /**
     * @return the DisplayManager
     */
    public DisplayManager getDisplayManager() {
        return displayManager;
    }

    /**
     * @return the HomeScreenController
     */
    public HomeScreenController getHomeScreenController() {
        return homeScreenController;
    }

    /**
     * @return the DialogController;
     */
    public DialogController getDialogController() {
        return dialogController;
    }

    public AccountScreenController getAccountScreenController() {
        return accountScreenController;
    }

    public void setAccountScreenController(AccountScreenController controller) {
        accountScreenController = controller;
    }

    public AccountScreenController getLoginScreenController() {
        return loginScreenController;
    }

    public void setLoginScreenController(AccountScreenController controller) {
        loginScreenController = controller;
    }

    public SignupScreenController getSignupScreenController() {
        return signupScreenController;
    }

    public void setSignupScreenController(SignupScreenController controller) {
        signupScreenController = controller;
    }

    public void setSyncSettingsScreenController(SyncSettingsScreenController syncSettingsScreenController) {
        this.syncSettingsScreenController = syncSettingsScreenController;
    }

    public void setAdvancedSettingsScreenController(AdvancedSettingsScreenController advancedSettingsScreenController) {
        this.advancedSettingsScreenController = advancedSettingsScreenController;
    }

    public Configuration getConfiguration() {
        return configuration;
    }

    // Misc interface functions

    public void toForeground() {
        Log.trace(TAG_LOG, "Application entered foreground");
        displayManager.toForeground();
    }

    public void toBackground() {
        Log.trace(TAG_LOG, "Application going to background");
        displayManager.toBackground();
    }

    public void reapplyConfiguration() {
        Log.trace(TAG_LOG, "Reapplying configuration");
        reapplyAccountConfiguration();
        reapplyUpdaterConfiguration();

        Enumeration appSources = BBAppSyncSourceManager.getInstance().getWorkingSources();
        while(appSources.hasMoreElements()) {
            BBAppSyncSource appSource = (BBAppSyncSource)appSources.nextElement();
            reapplySourceConfiguration(appSource);
        }

        reapplySyncModeConfiguration();
        reapplyMiscConfiguration();
    }


    public void reapplyAccountConfiguration() {
        // The account has changed, we shall propagate this information
        // to the sync source because they might want to know (for example the
        // photo source has the cache file whose name depends on the
        // credentials)
        Enumeration appSources = BBAppSyncSourceManager.getInstance().getWorkingSources();
        while(appSources.hasMoreElements()) {
            BBAppSyncSource appSource = (BBAppSyncSource)appSources.nextElement();
            appSource.reapplyConfiguration();
        }

        // Re-set the Updater url if enabled
        if(customization.enableUpdaterManager()) {
            String server = StringUtil.extractAddressFromUrl(configuration.getSyncUrl());
            UpdaterManager.getInstance().setUrl(server);
        }
    }

    public void reapplyUpdaterConfiguration() {
    }

    public void reapplySourceConfiguration(AppSyncSource appSource) {
        // Update the list of enabled sources
        homeScreenController.updateEnabledSources();
        appSource.reapplyConfiguration();
    }

    public void reapplySyncModeConfiguration() {
        // Handle the current selected SyncMode
        SyncModeHandler.handleCurrentSyncMode(this);

        // Note: C2S push does not need to be activated/deactivated. It always
        // listens to system changes, but it reads the config to know if syncs
        // shall be triggered
    }

    public boolean isVisible(AppSyncSource appSource) {
        if (appSource == null) {
            return false;
        }

        boolean visible = (appSource.isWorking() && appSource.getConfig().getActive());
        visible = visible && appSource.isVisible();
        visible = visible || customization.showNonWorkingSources();
        return visible;
    }

    public void reapplyMiscConfiguration() {
        // Re-set the log level
        Log.setLogLevel(configuration.getLogLevel());

        // Block incoming invites for events
        AppSyncSource calendarSource = appSyncSourceManager.getSource(AppSyncSourceManager.EVENTS_ID);
        if (calendarSource != null) {
            CalendarAppSyncSourceConfig config = (CalendarAppSyncSourceConfig)calendarSource.getConfig();
            client.setEnableBlockInvites(config.getBlockIncomingInvites());
        }
    }

    /**
     * Reapply server capabilities when the server sends them over. Sources can
     * be enabled/disabled or reconfigured based on the server capabilities
     * @param devInf the new server caps. It can be null if the server failed at
     * sending its caps. In this case the method uses default values
     */
    public void reapplyServerCaps(DevInf devInf) {

        Log.debug(TAG_LOG, "Reapply Server Caps: " + devInf);

        // Get the list of data stores
        Vector dataStores;
        if (devInf != null) {
            dataStores = devInf.getDataStores();
        } else {
            dataStores = new Vector();
        }

        // We have two sets of sources to compare. One is the set of sources
        // supported by this client, the other is the set of sources supported
        // by the server. We shall find a match for each source and
        // disable/configure everything which is supported
        Enumeration appSources = BBAppSyncSourceManager.getInstance().getRegisteredSources();

        while(appSources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)appSources.nextElement();
            // Search for a match for this source

            // Media sources are the ones that can appear/disappear depending on
            // the server configuration. All the other ones are always shown
            if (!appSource.getIsMedia()) {
                continue;
            }

            // Search a corresponding source in the list of sources supported by
            // the server
            String  sourceUri   = null;
            for(int i=0;i<dataStores.size();++i) {
                DataStore ds = (DataStore)dataStores.elementAt(i);
                Log.trace(TAG_LOG, "Found source with ref: " + ds.getSourceRef().getValue());
                String uri = appSource.getConfig().getUri();
                if (uri.equals(ds.getSourceRef().getValue())) {
                    sourceUri = ds.getSourceRef().getValue();
                    break;
                }
            }

            Log.trace(TAG_LOG, "Found source uri = " + sourceUri);

            AppSyncSourceConfig config = appSource.getConfig();
            if (sourceUri != null) {
                // Found it! enable the sync source (if it works)
                config.setUri(sourceUri);
                try {
                    Log.info(TAG_LOG, "Activating source " + appSource.getName() + "," + appSource.getId());
                    config.setActive(true);
                    config.setEnabled(true);

                    // Depending on the version of the server, we pick the
                    // best upload strategy
                    Vector exts = devInf.getExts();
                    boolean httpUpload = false;
                    if (exts != null) {
                        for(int j=0;j<exts.size();++j) {
                            Ext ext = (Ext)exts.elementAt(j);
                            if (Ext.X_FUNAMBOL_MEDIA_UPLOAD_HTTP.equals(ext.getXNam())) {
                                httpUpload = true;
                                break;
                            }
                        }
                    }
                    config.setUploadContentViaHttp(httpUpload);
                    reapplySourceConfiguration(appSource);
                } catch (Exception e) {
                    Log.error(TAG_LOG, "Source " + appSource.getName() + " not working, keep it disabled");
                }
            } else {
                // Disable the sync source
                Log.info(TAG_LOG, "Deactivating source " + appSource.getName() + "," + appSource.getId());
                config.setActive(false);
                config.setEnabled(false);
            }

            // Now recompute the ui position for all available sources
            int sources[] = customization.getSourcesOrder();
            int uiOrder = 0;
            for (int i=0;i<sources.length;++i) {
                int sourceId = sources[i];
                // If this is a working source, then set its UI position
                AppSyncSource source = BBAppSyncSourceManager.getInstance().getSource(sourceId);
                if (source != null) {
                    AppSyncSourceConfig sourceConfig = source.getConfig();
                    if (source.getSyncSource() != null && sourceConfig.getActive()) {
                        source.setUiSourceIndex(uiOrder++);
                    }
                }
            }
        }
        // Update the home screen
        homeScreenController.updateAvailableSources();
        // Update the configuration screen
        syncSettingsScreenController.updateListOfSources();
        // Clean the configuration flag that forces server caps to be asked
        configuration.setForceServerCapsRequest(false);
        // Save the configuration
        configuration.save();
    }

    /**
     * Called when user selected Download Update from the menu
     */
    public void promptUpdate() {
        UpdaterManager updater = UpdaterManager.getInstance();
        if (updater.isUpdateAvailable()) {
            synchronized (SyncClient.getEventLock()) {
                boolean result = dialogController.askYesNoQuestion(
                        localization.getLanguage("dialog_update"), true);
                if (result) {
                    displayManager.loadBrowser(updater.getDownloadUrl());
                } else {
                    dialogController.showMessage(
                            localization.getLanguage("message_update-location")
                            + ":\n" + UpdaterManager.getInstance().getDownloadUrl());
                }
            }
        }
    }

    public boolean checkForUpdate() {
        return checkForUpdate(true);
    }

    public boolean checkForUpdate(boolean canWait) {

        UpdaterManager updater = UpdaterManager.getInstance();

        if (customization.enableUpdaterManager()) {
            updater.setController(this);
            updater.check(canWait);
            if (updater.isUpdateAvailable()) {
                toForeground();
            }
        }
        return false;
    }

    public SourceConfig getSourceConfig(int x) {
        return client.getSourceConfig(x);
    }

    public int numErrors() {
        return errors.numErrors();
    }

    public void clearErrors() {
        errors.clearErrors();
    }

    public String getErrors() {
        return errors.getErrors();
    }

    public void error(String message) {
        errors.addError(message);
    }




    public void sendLogs() {

        Thread sendLogThread = new Thread() {

            public void run() {

                // We need to temporarily disable logging while we do this, so
                // that we dont capture the stuff logged during the send log

                int oldLevel = Log.getLogLevel();
                Log.setLogLevel(Log.ERROR);
                
                String logData = client.getLogContent();
                String alert = sendLogData(logData);
                
                Log.setLogLevel(oldLevel);
                getDialogController().showMessage(alert);
            }
        };

        sendLogThread.start();
    }

    public String sendLogData(String logData) {
        
        String username = configuration.getUsername();
        
        String result;
        if (logData == null) {
            result = localization.getLanguage("sendlog_no-logdata");
        } else if (username == null) {
            result = localization.getLanguage("sendlog_no-username");
        } else {

            String url = BBCustomization.getInstance().getSendLogsUri();
            String uri = url;
            // The uri is after the third /, http://server/uri
            for (int x = 0; x < 3; x++) {
                uri = uri.substring(uri.indexOf('/'));
            }
            HttpTransportAgent http = new HttpTransportAgent(url,
                    configuration.getSyncConfig().userAgent, "UTF-8", false, false);
            HttpDigestAuthentication auth = new HttpDigestAuthentication(username, configuration.getPassword(), uri);
            http.setAuthentication(auth);
            StringBuffer message = new StringBuffer();
            message.append("Username:").append(username).append("\n");
            message.append("Client:").append("Blackberry").append("\n");
            message.append("Device:").append(configuration.getSyncConfig().deviceConfig.devID).append("\n");
            message.append("OS:").append(BlackberryUtils.getSplittedOSVersion()).append("\n");
            message.append("Locale:").append(Locale.getDefaultForSystem().getCountry())
                    .append("\n");
            message.append("Model:").append(DeviceInfo.getDeviceName()).append("\n");
            message.append("Vendor:").append(Branding.getVendorId()).append("\n");
            message.append("Manufacturer:").append(DeviceInfo.getManufacturerName())
                    .append("\n");
            message.append("\n");
            message.append(logData);
            try {
                result = http.sendMessage(message.toString());
            } catch (Exception e) {
                result = localization.getLanguage("sendlog_cannot-send");
            }
        }
        return result;
    }
}
