/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.controller;

import java.util.Enumeration;
import java.util.Vector;
import net.rim.device.api.system.Bitmap;

import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.ui.HomeScreen;
import com.funambol.blackberry.ui.UISyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.blackberry.BBAppSyncSourceManager;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.blackberry.ui.DisplayManager;
import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.util.Log;

/**
 * This class represents the controller for the home screen. Since the
 * HomeScreen is a screen where synchronizations can be performed, the
 * class extends the SynchronizationController. On top of this the class adds
 * the ability of handling the home screen.
 */
public class HomeScreenController extends SynchronizationController {

    private static final String  TAG_LOG = "HomeScreenController";
    private static final String  IMAGE_ARROWS_IDLE       = "arrows1.png";
    private HomeScreen           homeScreen;

    private UISyncSource         syncAllItem = null;

    private Vector               items = null;

    private boolean              updateAvailableSources = false;

    private boolean              syncTypeWarningDisplayed = false;

    HomeScreenController(Controller controller, HomeScreen homeScreen, SyncClient client) {
        super(controller, client);

        // We set ourselves as listener for the SyncEngine so we get notified on
        // events
        engine.setListener(this);

        this.controller = controller;
        this.homeScreen = homeScreen;
        forceUpdateAvailableSources();
    }

    public Controller getController() {
        return controller;
    }

    public boolean syncStarted(Vector sources) {
        Log.trace(TAG_LOG, "syncStarted");

        lockHomeScreen(sources);

        if (customization.syncAllOnMainScreenRequired()) {
            if (syncAllItem != null) {
                syncAllItem.setTitle(localization.getLanguage("status_sync"));
            } else {
                Log.error(TAG_LOG, "Program error, syncAllItem should not be null");
            }
        }

        homeScreen.disableListeners();

        if (customization.reportStatusInSyncAll() && syncAllItem != null) {
            syncAllItem.setStatusString(localization.getLanguage("sync_check-update"));
        }

        boolean res = super.syncStarted(sources);

        if (customization.reportStatusInSyncAll() && syncAllItem != null) {
            syncAllItem.setStatusString(null);
        }
        return res;
    }

    public void updateAvailableSources() {
        Log.trace(TAG_LOG, "updateAvailableSources");
        updateAvailableSources = true;
    }

    private void forceUpdateAvailableSources() {
        Log.trace(TAG_LOG, "forceUpdateAvailableSources");
        // Compute the set of items to be displayed
        computeVisibleItems();
        homeScreen.updateVisibleItems(getFirstActiveItemIndex());
        updateAvailableSources = false;
    }

    public void syncEnded() {
        super.syncEnded();

        homeScreen.enableListeners();
        homeScreen.setSelectedIndex(getFirstActiveItemIndex());
        if (customization.syncAllOnMainScreenRequired()) {
            if (syncAllItem != null) {
                syncAllItem.setStatusString(null);
                syncAllItem.setTitle(localization.getLanguage("home_sync-all"));
            }
        }
        unlockHomeScreen();
    }

    public void redraw() {

        // We may need to update the list of
        // visible items as the server may have sent its capabilities
        if (updateAvailableSources) {
            forceUpdateAvailableSources();
        }

        homeScreen.redraw();
    }

    public Vector getVisibleItems() {
        return items;
    }

    public void buttonPressed(int index) {
        Log.trace(TAG_LOG, "buttonPressed " + index);
        
        if (customization.syncAllOnMainScreenRequired()) {
            if (index != 0) {
                AppSyncSource source = (AppSyncSource) items.elementAt(index-1);
                syncMenuSelected(source);
            } else {
                syncMenuSelected();
            }
        } else {
            AppSyncSource source = (AppSyncSource) items.elementAt(index);
            if (source.getConfig().getEnabled()) {
               syncMenuSelected(source);
            } else {
               syncMenuSelected();
            }
        }
    }

    public void sourceStarted(AppSyncSource appSource) {
        super.sourceStarted(appSource);
        // this selects the appSource and disable any previously selected one
        int index = appSource.getUiSourceIndex();
        if (customization.syncAllOnMainScreenRequired()) {
            index++;
        }
        homeScreen.setSelectedIndex(index);
    }

    public void sourceEnded(AppSyncSource appSource) {
        super.sourceEnded(appSource);
    }

    private void lockHomeScreen(Vector sources) {

        if (customization.syncAllOnMainScreenRequired()) {
            // disable the sync all button
            syncAllItem.setEnabled(false);
        }

        for(int j=0;j<items.size();++j) {
            BBAppSyncSource appSource = (BBAppSyncSource) items.elementAt(j);
            // If this source is in sources then we shall enable it,
            // otherwise we must disable it
            boolean enable = false;
            for(int i=0;i<sources.size();++i) {
                AppSyncSource appSource2 = (AppSyncSource)sources.elementAt(i);
                if (appSource2.getId() == appSource.getId()) {
                    enable = true;
                    break;
                }
            }
            UISyncSource uiSource = appSource.getClientUISyncSource();
            uiSource.setEnabled(enable);
        }
        redraw();
    }

    public void updateEnabledSources() {

        Log.trace(TAG_LOG, "updateEnabledSources");

        // If a sync is in progress, then we don't change the sources status,
        // otherwise we would corrupt the UI. On sync termination, the home
        // screen will get refreshed
        if (isSynchronizing()) {
            return;
        }

        unlockHomeScreen();

        Enumeration sources = items.elements();
        while (sources.hasMoreElements()) {
            BBAppSyncSource appSource = (BBAppSyncSource)sources.nextElement();
            UISyncSourceController sourceController = appSource.getClientUISyncSourceController();

            if (sourceController != null) {

                if (!appSource.getConfig().getEnabled()) {
                    sourceController.disable();
                    UISyncSource uiSource = appSource.getClientUISyncSource();
                    // If this is the selected source, then we shall move the
                    // selection to the first available
                    if (uiSource != null && uiSource.isSelected()) {
                        homeScreen.setSelectedIndex(getFirstActiveItemIndex());
                    }
                } else {
                    sourceController.enable();
                }
            }
        }

        redraw();
    }

    public void exposed() {
        // When the home screen is exposed, we need to update the status of each
        // ui source (only if there is not a synchronization in progress)
        if (!isSynchronizing()) {
            for(int i=0;i<items.size();++i) {
                BBAppSyncSource source = (BBAppSyncSource)items.elementAt(i);
                UISyncSourceController sourceController = source.getClientUISyncSourceController();
                sourceController.refreshStatus();
            }
        }
        // Check if we must show the warning about the sync type migration we 
        // did in v9
        if(configuration.getPimSourceSyncTypeChanged() && !syncTypeWarningDisplayed) {
            showPimSyncTypeChangedWarning();
        }
    }

    protected void showPimSyncTypeChangedWarning() {
        new SyncTypeWarningThread().start();
    }

    public void syncMenuSelected(AppSyncSource appSource) {
        Vector sources = new Vector();
        sources.addElement(appSource);
        synchronize(com.funambol.client.controller.SynchronizationController.MANUAL, sources);
    }

    public void syncMenuSelected() {
        Vector sources = new Vector();
        for(int i=0;i<items.size();++i) {
            AppSyncSource appSource = (AppSyncSource)items.elementAt(i);
            if (appSource.getConfig().getEnabled() && appSource.isWorking()) {
                sources.addElement(appSource);
            }
        }
        synchronize(com.funambol.client.controller.SynchronizationController.MANUAL, sources);
    }


    public void refreshMenuSelected() {

        if(customization.enableUpdaterManager() &&
                UpdaterManager.getInstance().needsToUpdate()){
            Log.info(TAG_LOG, "Needs to upgrade application");
            UpdaterManager updMan = UpdaterManager.getInstance();
            updMan.setController(controller);
            updMan.promptExpiredUpdate();
            return;
        }

        if (!controller.getDialogController().promptNext(
                localization.getLanguage("dialog_refresh_warn"))) {
            return;
        }

        int direction = controller.getDialogController().showRefreshDirectionDialog();
        if (direction >= 0) {
            int mask = controller.getDialogController().showRefreshTypeDialog(direction);
            if (mask > 0) {
                controller.getScreenDisplayController().showHomeScreen();
                refresh(mask, direction);
            }
        }

    }

    public void cancelMenuSelected() {
        cancelSync();
    }

    public void updateMenuSelected() {
        controller.promptUpdate();

    }

    public void aboutMenuSelected() {
        controller.getScreenDisplayController().showAboutScreen();
    }

    public void quitMenuSelected() {
        controller.toBackground();
    }

    public void configMenuSelected() {
        controller.getScreenDisplayController().showConfigScreen();
    }

    public void logoutMenuSelected() {
        // Logout
        configuration.setCredentialsCheckPending(true);
        configuration.save();
        controller.getScreenDisplayController().hideHomeScreen();
        controller.getScreenDisplayController().showLoginScreen();
    }

    public void errorMenuSelected() {
        controller.getDialogController().showMessage(getErrors());
    }

    public int numErrors() {
        return controller.numErrors();
    }

    public void sendLogs() {
        controller.sendLogs();
    }

    private void unlockHomeScreen() {

        if (customization.syncAllOnMainScreenRequired()) {
            // enable the sync all button
            syncAllItem.setEnabled(true);
        }

        for(int j=0;j<items.size();++j) {
            BBAppSyncSource appSource = (BBAppSyncSource) items.elementAt(j);
            // If this source is in sources then we shall enable it,
            // otherwise we must disable it
            UISyncSource uiSource = appSource.getClientUISyncSource();
            uiSource.setEnabled(appSource.getConfig().getEnabled());
        }
        redraw();
    }

    private String getErrors() {
        return controller.getErrors();
    }

    private int getFirstActiveItemIndex() {
        if (customization.syncAllOnMainScreenRequired()) {
            return 0;
        }

        int size = items.size();
        
        for (int i=0; i<size; i++) {
            AppSyncSource source = (AppSyncSource) items.elementAt(i);
            if (source.getConfig().getEnabled() && source.isWorking()) {
                return i;
            }
        }   
        return 0;
    }

    private void computeVisibleItems() {

        items = new Vector();

        Bitmap bg, bgSel;
        UISyncSource item;

        //Set the SyncAll Item if required
        if (customization.syncAllOnMainScreenRequired()) {
            String imageAll = (String)customization.getSyncAllIcon().getOpaqueDescriptor();

            String imageAllBg = (String)customization.getSyncAllBackground().getOpaqueDescriptor();
            bg = Bitmap.getBitmapResource(imageAllBg);

            String imageAllBgHigh = (String)customization.getSyncAllHighlightedBackground().getOpaqueDescriptor();
            bgSel = Bitmap.getBitmapResource(imageAllBgHigh);

            syncAllItem = homeScreen.createSyncAllItem(localization.getLanguage("home_sync-all"),
                                                       null, imageAll, imageAll, bg, bgSel);

            syncAllItem.setStatusSelectedIcon(Bitmap.getBitmapResource(IMAGE_ARROWS_IDLE));

            syncAllItem.setSelected(true);
            syncAllItem.setTitleEnabledColor(customization.getSyncAllTextColor());
            syncAllItem.setTitleDisabledColor(customization.getSyncAllTextColor());
            syncAllItem.setEnabledStatusColor(customization.getSyncAllTextColor());
            syncAllItem.setDisabledStatusColor(customization.getSyncAllTextColor());
            syncAllItem.setContainer((UISyncSourceContainer)homeScreen);
        }

        String bgName = (String)customization.getButtonBackground().getOpaqueDescriptor();
        String bgSelName = (String)customization.getButtonHighlightedBackground().getOpaqueDescriptor();
        bg = Bitmap.getBitmapResource(bgName);
        bgSel = Bitmap.getBitmapResource(bgSelName);

        int offset = items.size();
        // TODO: remove one for the config, but this should not be hardcoded
        items.setSize(BBAppSyncSourceManager.getInstance().numberOfWorkingSources() + offset - 1);

        // Add an item for each registered source that has to fit into the home
        // screen. So far the only one we shall discard is the ConfigSyncSource
        Enumeration sources = BBAppSyncSourceManager.getInstance().getWorkingSources();
        while (sources.hasMoreElements()) {
            AppSyncSource appSource = (AppSyncSource)sources.nextElement();
            AppSyncSourceConfig appConfig = appSource.getConfig();

            if (appConfig.getActive() && appSource.getId() != BBAppSyncSourceManager.SYNC_FLAG_CONFIG) {
                // Set the sources in the appropriate order
                int index = appSource.getUiSourceIndex();
                items.setElementAt(appSource, index + offset);
            }
        }
    }

    private class SyncTypeWarningThread extends Thread {
        public void run() {
            syncTypeWarningDisplayed = true;
            Log.debug(TAG_LOG, "Showing pim sync type changed warning");
            DisplayManager dm = controller.getDisplayManager();
            String[] buttons = new String[] { localization.getLanguage("dialog_ok") };
            int[] values = new int[] { 0 };
            int response = dm.promptSelection(localization.getLanguage(
                    "upg_one_way_no_longer_supported"),
                    buttons, values, 0, true, DisplayManager.NO_LIMIT);
            if(response == 0) {
                configuration.setPimSourceSyncTypeChanged(false);
                configuration.save();
            }
        }
    }
}
