/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2009 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.media;

import java.io.IOException;
import java.util.Vector;

import com.funambol.blackberry.BBFileSystemListener;
import com.funambol.blackberry.source.ExtendedSyncSource;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.client.controller.SynchronizationController;
import com.funambol.syncml.protocol.SyncML;
import com.funambol.syncml.spds.SyncSource;

import com.funambol.client.ui.Screen;
import com.funambol.client.ui.SettingsUISyncSource;
import com.funambol.util.Log;

public abstract class MediaAppSyncSource extends BBAppSyncSource {

    private static final String TAG_LOG = "MediaAppSyncSource";

    private Vector extensions = null;

    private String internalMemoryMediaDir;
    private String sdCardMediaDir;

    public MediaAppSyncSource(String name, SyncSource source) {
        super(name, source);
    }

    public MediaAppSyncSource(String name) {
        super(name);
    }

    public int prepareRefresh(int direction) {
        int syncMode = 0;
        switch (direction) {
            case SynchronizationController.REFRESH_FROM_SERVER:
                throw new IllegalArgumentException("Invalid refresh direction " + direction);
            case SynchronizationController.REFRESH_TO_SERVER:
                syncMode = SyncML.ALERT_CODE_ONE_WAY_FROM_CLIENT_NO_SLOW;
                break;
            default:
                throw new IllegalArgumentException("Invalid refresh direction " + direction);
        }
        // Now clear the tracker so that everything is sent again to the server
        if(source instanceof ExtendedSyncSource) {
            try {
                ExtendedSyncSource ess = (ExtendedSyncSource)source;
                ess.clearTracker();
            } catch (IOException ioe) {
                Log.error(TAG_LOG, "Cannot clear tracker", ioe);
                throw new IllegalArgumentException("Cannot refresh a source whose tracker cannot be cleared");
            }
        } 
        return syncMode;
    }

    public boolean isRefreshSupported(int direction) {

        boolean supported;
        switch (direction) {
            case SynchronizationController.REFRESH_FROM_SERVER:
                supported = false;
                break;
            case SynchronizationController.REFRESH_TO_SERVER:
                supported = true;
                break;
            default:
                throw new IllegalArgumentException("Invalid refresh direction " + direction);
        }
        return supported;
    }

    public void setRefreshSupported(boolean value) {
        throw new IllegalArgumentException("Media sync source refresh hanlding cannot be changed");
    }

    public void setRefreshSupported(int direction, boolean value) {
        throw new IllegalArgumentException("Media sync source refresh hanlding cannot be changed");
    }

    public void reapplyConfiguration() {

        super.reapplyConfiguration();
        
        String picturesDir = getMediaDir();
        // Set this property in the SS and in the system listener
        BBFileSystemListener.BBFileSystemListenerDirectory listener = (BBFileSystemListener.BBFileSystemListenerDirectory)getNativeListener();
        if (listener != null) {
            listener.setDirectory(picturesDir);
        }
        if(source instanceof ExtendedSyncSource) {
            ExtendedSyncSource ss = (ExtendedSyncSource)getSyncSource();
            ss.reapplyConfiguration();
        }
    }

    /**
     * Returns the message to show at the very first sync for this source.
     * For the media sync source we only want to show the msg if the user is
     * about to sync also old media.
     */
    public String getWarningOnFirstSync() {
        if (getMediaSyncOld()) {
            return warningOnFirstSync;
        } else {
            return null;
        }
    }

    public SettingsUISyncSource createSettingsUISyncSource(Screen screen) {
        SettingsUISyncSource settings = super.createSettingsUISyncSource(screen);
        // The media source settings requires the controller
        if (settings instanceof MediaSourceSettings) {
            MediaSourceSettings mSettings = (MediaSourceSettings)settings;
            mSettings.setController(controller);
        }
        return settings;
    }

    public void setExtensions(String extensions[]) {
        this.extensions = new Vector();
        for(int i=0;i<extensions.length;++i) {
            this.extensions.addElement(extensions[i]);
        }
    }

    public void setExtensions(Vector extensions) {
        this.extensions = extensions;
    }

    public Vector getExtensions() {
        return extensions;
    }

    public String getSDCardMediaDir() {
        return sdCardMediaDir;
    }

    public void setSDCardMediaDir(String sdCardMediaDir) {
        this.sdCardMediaDir = sdCardMediaDir;
    }

    public String getInternalMemoryMediaDir() {
        return internalMemoryMediaDir;
    }

    public void setInternalMemoryMediaDir(String dir) {
        this.internalMemoryMediaDir = dir;
    }

    protected String getMediaDir() {
        MediaAppSyncSourceConfig appConfig = (MediaAppSyncSourceConfig)getConfig();
        return appConfig.getDirectory();
    }

    protected boolean getMediaSyncOld() {
        MediaAppSyncSourceConfig appConfig = (MediaAppSyncSourceConfig)getConfig();
        return appConfig.getIncludeOld();
    }
}


