/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.source.media;

import com.funambol.blackberry.BBLocalization;
import java.util.Vector;

import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.component.ObjectChoiceField;

import javax.microedition.io.file.FileSystemRegistry;
import javax.microedition.io.file.FileSystemListener;

import com.funambol.blackberry.ui.rim.component.ColoredLabelField;
import com.funambol.blackberry.ui.rim.RimSourceBasicSettings;
import com.funambol.blackberry.ui.rim.RimFileBrowser;
import com.funambol.blackberry.ui.rim.RimFileBrowserListener;

import com.funambol.client.configuration.Configuration;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.container.HorizontalFieldManager;

/**
 * This class creates the configuration screen for configuring parameters
 * required for the SyncClient. In particular it supports the advanced settings
 * and the sync settings.
 */
public class MediaSourceSettings extends RimSourceBasicSettings {

    private static final String TAG_LOG = "MediaSourceSettings";

    private static final String[] DIR_LABEL = {
        BBLocalization.getInstance().getLanguage("conf_pictures_phone_storage"),
        BBLocalization.getInstance().getLanguage("conf_pictures_sdcard_storage"),
        BBLocalization.getInstance().getLanguage("conf_pictures_custom_storage")
    };

    private static final String[] DIR_LABEL_NO_SDCARD = {
        BBLocalization.getInstance().getLanguage("conf_pictures_phone_storage"),
        BBLocalization.getInstance().getLanguage("conf_pictures_custom_storage")
    };

    private ColoredLabelField dirLabelField         = null;
    private MediaLocationChoiceField locationChoiceField = null;
    private CheckboxField syncOld                   = null;

    private String currentDir                       = null;

    private static FSListener fsListener            = null;

    public MediaSourceSettings() {
        super();

        dirLabelField = new ColoredLabelField("");
        locationChoiceField = new MediaLocationChoiceField();
        String locationLabel = BBLocalization.getInstance().getLanguage("conf_location_label");
        locationChoiceField.setLabel(locationLabel);

        // The sync source name is not available here
        syncOld = new CheckboxField("", false);

        // We have a specialized listener
        MediaCheckBoxListener ml = new MediaCheckBoxListener(enableField);
        enableField.setChangeListener(null);
        enableField.setChangeListener(ml);

        syncOld.setChangeListener(cl);
    }

    public void layout() {
        // First of all the standard settings
        super.layout();
        // Now add the media settings (sync old and media location)
        String syncOldLabel = BBLocalization.getInstance().getLanguage("conf_sync_old");
        String name = source.getName().toLowerCase();
        syncOldLabel = StringUtil.replaceAll(syncOldLabel, "__SOURCE__", name);
        syncOld.setLabel(syncOldLabel);

        MediaAppSyncSourceConfig config = (MediaAppSyncSourceConfig)source.getConfig();

        reinitializeDirOptions();

        if (!config.getEnabled()) {
            dirLabelField.setColor(Color.GRAY);
            locationChoiceField.setEditable(false);
        }

        // Rebuild vertical manager
        vfm.deleteAll();

        if (enableLabel != null) {
            HorizontalFieldManager title = new HorizontalFieldManager();

            StringBuffer oneWayLabel = new StringBuffer(" ");
            oneWayLabel.append(localization.getLanguage("media_upload_only_info"));
            LabelField oneWayInfo = new ColoredLabelField(oneWayLabel.toString(),
                    Field.FIELD_BOTTOM);
            title.add(enableLabel);
            title.add(oneWayInfo);
            vfm.add(title);
        }
        if (uriField != null) {
            vfm.add(uriField);
        }

        vfm.add(locationChoiceField);
        vfm.add(dirLabelField);
        vfm.add(syncOld);

        // Start monitoring FS roots
        if (fsListener == null) {
            // We need to register a FS listener to be notified when the SDCard is
            // plugged/unplugged
            fsListener = new FSListener();
            FileSystemRegistry.addFileSystemListener(fsListener);
        }

        fsListener.start(this);
    }

    public void saveSettings(Configuration configuration) {
        if (source != null) {
            super.saveSettings(configuration);
            Log.trace(TAG_LOG, "Saving settings for " + source.getName());
            MediaAppSyncSourceConfig appConfig = (MediaAppSyncSourceConfig)source.getConfig();
            // Current dir is always without protocol specifier
            appConfig.setDirectory("file://" + currentDir);
            appConfig.setIncludeOld(syncOld.getChecked());
        }
        dirty = false;
    }

    public void loadSettings(Configuration configuration) {
        if (source != null) {
            super.loadSettings(configuration);
            Log.trace(TAG_LOG, "Loading settings for " + source.getName());
            MediaAppSyncSourceConfig appConfig = (MediaAppSyncSourceConfig)source.getConfig();
            currentDir = appConfig.getDirectory();
            // Current dir is always without protocol specifier
            if (currentDir == null) {
                // This should never happen, but just to be super safe...
                MediaAppSyncSource mediaSource = (MediaAppSyncSource)source;
                currentDir = mediaSource.getInternalMemoryMediaDir();
            }
            currentDir = currentDir.substring(7);
            Log.trace(TAG_LOG, "currentDir=" + currentDir);
            syncOld.setChecked(appConfig.getIncludeOld());
        }
    }
 

    private void reinitializeDirOptions() {

        if (source == null) {
            return;
        }
        Log.trace(TAG_LOG, "reinitialize dir options " + currentDir);

        // This is the case where the picture sync source is not active
        if (locationChoiceField == null) {
            return;
        }

        String dirSelectionName[];
        if (BlackberryUtils.isSDCardInserted()) {
            dirSelectionName = DIR_LABEL;
        } else {
            dirSelectionName = DIR_LABEL_NO_SDCARD;
        }

        locationChoiceField.setChoices(dirSelectionName);


        boolean isSDcardInserted = BlackberryUtils.isSDCardInserted();

        MediaAppSyncSource mediaSource = (MediaAppSyncSource)source;
        String sdCardDir = mediaSource.getSDCardMediaDir().substring(7);
        String internalDir = mediaSource.getInternalMemoryMediaDir().substring(7);
        boolean isSDCardSelected = currentDir.equals(sdCardDir);
        boolean isInternalMemorySelected = currentDir.equals(internalDir);

        Log.trace(TAG_LOG, "SDCardMediaDir=" + sdCardDir);
        Log.trace(TAG_LOG, "InternalMemoryMediaDir=" + internalDir);

        if (isSDcardInserted) {
            //SDCARD is inserted
            if (isInternalMemorySelected) {
                locationChoiceField.setSelectedIndex(0);
                // Hide default values
                dirLabelField.setText("");
            } else if (isSDCardSelected) {
                locationChoiceField.setSelectedIndex(1);
                // Hide default values
                dirLabelField.setText("");
            } else {
                locationChoiceField.setSelectedIndex(2);
                dirLabelField.setText(currentDir);
            }
        } else {
            //SDCARD is NOT inserted
            if (isInternalMemorySelected) {
                locationChoiceField.setSelectedIndex(0);
                // Hide default values
                dirLabelField.setText("");
            } else {
                locationChoiceField.setSelectedIndex(1);
                dirLabelField.setText(currentDir);
            }
        }
    }


    private class MediaLocationChoiceField extends ObjectChoiceField {

        private int previousIndex;

        public MediaLocationChoiceField() {
            super();
        }

        public int getPreviousIndex() {
            return previousIndex;
        }

        protected boolean trackwheelClick(int status, int time) {
            // Mark the configuration as having changes at this point. This is
            // not optimal, but it is conservative
            dirty = true;
            // Remember the current index
            previousIndex = getSelectedIndex();
            return super.trackwheelClick(status, time);
        }

        protected void setSelectedIndex(int index, int context) {
            super.setSelectedIndex(index, context);
            if (context == 2) {
                // This is the case where the user clicked on the item to select
                // it
                MediaAppSyncSource mediaSource = (MediaAppSyncSource)source;
                switch(index) {
                    case 0:
                        currentDir = mediaSource.getInternalMemoryMediaDir().substring(7);
                        dirLabelField.setText("");
                        break;
                    case 1:
                        if (BlackberryUtils.isSDCardInserted()) {
                            currentDir = mediaSource.getSDCardMediaDir().substring(7);
                            dirLabelField.setText("");
                        } else {
                            showFileBrowser();
                        }
                        break;
                    case 2:
                        showFileBrowser();
                        break;
                    default:
                        break;
                }
            }
        }

        private void showFileBrowser() {
            MediaAppSyncSource mediaSource = (MediaAppSyncSource)source;
            Vector extensions = mediaSource.getExtensions();

            int previousIndex = locationChoiceField.getPreviousIndex();
            MediaFileBrowserListener listener = new MediaFileBrowserListener(previousIndex, currentDir);
            if (controller != null) {
                RimFileBrowser fileBrowser = new RimFileBrowser(controller, RimFileBrowser.ALL_MODE,
                        RimFileBrowser.SELECT_DIR_MODE, extensions);
                // Set the initial path in the browser. Note that this
                // path does not contain the "file://" prefix
                fileBrowser.setCaseSensitive(false);
                fileBrowser.setPath(currentDir);
                fileBrowser.setListener(listener);
                try {
                    controller.getScreenDisplayController().showScreen(fileBrowser);
                } catch (Exception e) {
                    Log.error(TAG_LOG, "Error while showing file browser ", e);
                }
            } else {
                Log.error(TAG_LOG, "Cannot open browser because no controller was provided");
            }
        }
    }

    class MediaCheckBoxListener extends RimSourceBasicSettings.ChangeListener {

        private CheckboxField  field;

        public MediaCheckBoxListener(CheckboxField field) {
            this.field = field;
        }

        public void fieldChanged(Field valueField, int context) {

            super.fieldChanged(valueField, context);

            // This may happen the very first time, before the layout method is
            // invoked
            if (locationChoiceField == null || syncOld == null) {
                return;
            }

            if (field.getChecked()) {
                // Pictures sync is enabled, we shall enable the browse button
                locationChoiceField.setEditable(true);
                syncOld.setEditable(true);
            } else {
                // Pictures sync is disabled, we shall disable the browse button
                locationChoiceField.setEditable(false);
                syncOld.setEditable(false);
            }
        }
    }

    private class MediaFileBrowserListener extends RimFileBrowserListener {

        private int previousIndex;
        private String previousDir;
        public MediaFileBrowserListener(int previousIndex, String previousDir) {
            this.previousIndex = previousIndex;
            this.previousDir   = previousDir;
        }

        public boolean chosen(Vector entries) {
            // We expect a single selection here
            if (entries.size() != 1) {
                return false;
            }
            String dir = (String)entries.elementAt(0);
            // Set the new directory in the Configuration
            Log.debug(TAG_LOG, "Setting media dir " + "file://" + dir);

            // Update the label with the current value
            dirLabelField.setText(dir);
            currentDir = dir;
            return true;
        }

        public boolean cancelled() {
            // Restore the previous values
            MediaAppSyncSource mediaSource = (MediaAppSyncSource)source;
            String internalDir = mediaSource.getInternalMemoryMediaDir().substring(7);
            boolean done = false;
            if (internalDir.equals(previousDir)) {
                dirLabelField.setText("");
                done = true;
            }
            if (BlackberryUtils.isSDCardInserted()) {
                String sdDir = mediaSource.getSDCardMediaDir().substring(7);
                if (sdDir.equals(previousDir)) {
                    dirLabelField.setText("");
                    done = true;
                }
            }
            if (!done) {
                dirLabelField.setText(previousDir);
            }
            locationChoiceField.setSelectedIndex(previousIndex);
            currentDir = previousDir;
            return true;
        }
    }

    private class FSListener implements FileSystemListener {
        private Vector listeners = new Vector();

        public FSListener() {
        }

        public void start(MediaSourceSettings settings) {
            listeners.addElement(settings);
        }

        public void rootChanged(int state, String rootName) {
            // We could check if this is the SDCard, but at the moment we always
            // refresh
            for(int i=0;i<listeners.size();++i) {
                Log.debug(TAG_LOG, "RootChanged " + rootName);
                MediaSourceSettings s = (MediaSourceSettings)listeners.elementAt(i);
                s.reinitializeDirOptions();
            }
        }
    }

}


