/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2008 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.system.Bitmap;
import net.rim.device.api.system.EncodedImage;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.FontFamily;
import net.rim.device.api.math.Fixed32;

import javax.microedition.lcdui.Image;

import com.funambol.blackberry.BBCustomization;

import tube42.lib.ui.ImageUtils;

import com.funambol.util.Log;


public class RimHelper {

    private static final String TAG_LOG = "RimHelper";

    public static Font getDefaultFont() {
    
        Font font = Font.getDefault();
        if (! BBCustomization.getInstance().useDefaultFont()) {
            try {
                FontFamily ff = null;
                String fontName = BBCustomization.getInstance().getPreferredFont();
                if (fontName != null) {
                    ff = FontFamily.forName(fontName);
                    font = ff.getFont(BBCustomization.getInstance().getFontStyle(),
                                      BBCustomization.getInstance().getFontSize());
                }
            } catch (ClassNotFoundException e) {
            }
        }
        
        return font;
    }

    public static Bitmap getAndResizeImage(String name, int width, int height) {
        EncodedImage ei = EncodedImage.getEncodedImageResource(name);  
        if (ei != null) {
            ei = resizeImage(ei, width, height);
            return ei.getBitmap();
        } else {
            return null;
        }
    }

    public static EncodedImage resizeImage(EncodedImage image, int width, int height) {
        EncodedImage result = null;

        int currentWidthFixed32 = Fixed32.toFP(image.getWidth());
        int currentHeightFixed32 = Fixed32.toFP(image.getHeight());

        int requiredWidthFixed32 = Fixed32.toFP(width);
        int requiredHeightFixed32 = Fixed32.toFP(height);

        int scaleXFixed32 = Fixed32.div(currentWidthFixed32, requiredWidthFixed32);
        int scaleYFixed32 = Fixed32.div(currentHeightFixed32, requiredHeightFixed32);

        result = image.scaleImage32(scaleXFixed32, scaleYFixed32);
        return result;
    }

    /**
     * This method tries to decode the image and resize it with anti aliasing.
     * In general the quality of the generated image is much better than what we
     * get with the getAndResizeImage, but the method may run out of memory and
     * it is not fast.
     * If during the conversion there is an out of memory problem, then we
     * revert to the resizing without AA.
     */
    public static Bitmap getAndResizeImageWithAA(String name, int width, int height) {
        Bitmap bmp = Bitmap.getBitmapResource(name);
        try {
            if (bmp != null) {
                int bmpWidth = bmp.getWidth();
                int bmpHeight = bmp.getHeight();
                int data[] = new int[bmpWidth * bmpHeight];
                bmp.getARGB(data, 0, bmp.getWidth(), 0, 0, bmpWidth, bmpHeight);
                // Allow the GC to pick this memory
                bmp = null;
                Image img = Image.createRGBImage(data, bmpWidth, bmpHeight, true);
                // Allow the GC to pick this memory
                data = null;
                img = ImageUtils.resize(img, width, height, true, false);
                bmp = new Bitmap(width, height);
                // Recreate the data array
                data = new int[width * height];
                img.getRGB(data, 0, width, 0, 0, width, height);
                bmp = new Bitmap(width, height);
                bmp.setARGB(data, 0, width, 0, 0, width, height);
                data = null;
            }
        } catch (Exception e) {
            Log.error(TAG_LOG, "Cannot get image with anti aliasing", e);
            bmp = getAndResizeImage(name, width, height);
        }
        return bmp;
    }






}
