/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import java.util.Vector;

import net.rim.blackberry.api.mail.Session;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.component.Dialog;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.MainScreen;

import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBAppSyncSourceManager;
import com.funambol.blackberry.BBLocalization;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.ProxyController;
import com.funambol.blackberry.controller.BBAdvancedSettingsScreenController;

import com.funambol.client.ui.AdvancedSettingsScreen;
import com.funambol.client.ui.SyncSettingsScreen;
import com.funambol.client.ui.SettingsUISyncSource;
import com.funambol.client.ui.SettingsUIItem;
import com.funambol.client.ui.ConfigurationScreen;
import com.funambol.client.controller.SyncSettingsScreenController;
import com.funambol.client.localization.Localization;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.Log;
import com.funambol.util.FileAppender;


/**
 * This class creates the configuration screen for configuring parameters
 * required for the SyncClient. In particular it supports the advanced settings
 * and the sync settings.
 */
public class RimSettingsScreen extends MainScreen implements
        AdvancedSettingsScreen, SyncSettingsScreen, ConfigurationScreen {

    private static final String TAG_LOG = "RimSettingsScreen";

    private Vector sourceSettingList;

    /**
     * This creates a MenuItem containing "Save" option. on selecting the save
     * option the configuration details are saved.
     */
    private final MenuItem saveMenu      = new MenuItem(BBLocalization.getInstance()
                                                  .getLanguage("menu_save"),
                                                  20000,
                                                  10) {
                                              public void run() {
                                                  if (saveConfiguration()) {
                                                      controller.getScreenDisplayController()
                                                              .hideConfigScreen();
                                                  }
                                              }
                                          };

    private final MenuItem  backMenu     = new MenuItem(BBLocalization.getInstance()
                                                  .getLanguage("menu_back"),
                                                  20000,
                                                  10) {
                                              public void run() {
                                                  onClose();
                                              }
                                          };

    private final MenuItem  sendLogMenu  = new MenuItem(BBLocalization.getInstance()
                                                  .getLanguage("menu_sendlog"),
                                                  20000,
                                                  10) {
                                              public void run() {
                                                  advSettingsController.sendLog();
                                              }
                                           };

    private final Controller controller;

    private BBConfiguration configuration = BBConfiguration.getInstance();
    private BBCustomization customization = BBCustomization.getInstance();
    private BBAppSyncSourceManager appSyncSourceManager = BBAppSyncSourceManager.getInstance();

    private boolean dirty                          = true;
    private boolean enableSendLog                  = true;
    private RimSyncModeSetting modeSetting         = null;
    private RimLogSetting logSetting               = null;
    private SyncSettingsScreenController settingsController = null;
    private BBAdvancedSettingsScreenController advSettingsController = null;

    /**
     * The constructor here creates the Configuration screen with all the UI
     * components on it and sets default input minValues
     */
    public RimSettingsScreen(Controller ctrl) {

        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT | Field.FIELD_LEFT | Field.FIELD_TOP);

        Log.trace(TAG_LOG, "RimSettingsScreen");
        this.setFont(RimHelper.getDefaultFont());
        controller = ctrl;

        // Set the screen title
        LabelField title = new LabelField(BBLocalization.getInstance().getLanguage("conf_title"));
        title.setFont(getFont().derive(RimTheme.TITLE_FONT_STYLE,
                RimTheme.TITLE_FONT_SIZE));
        setTitle(title);

        settingsController = new SyncSettingsScreenController(ProxyController.getInstance(),
                                                              customization, configuration,
                                                              appSyncSourceManager, this);
        ctrl.setSyncSettingsScreenController(settingsController);

        Localization localization = BBLocalization.getInstance();

        advSettingsController = new BBAdvancedSettingsScreenController(ProxyController.getInstance(),
                                                                       customization,
                                                                       configuration,
                                                                       localization,
                                                                       appSyncSourceManager,
                                                                       this,
                                                                       controller.getDialogController());

        // By default we have a log setting. So we initially create it. If it
        // must be hidden, then the advSettingsController will hide it during
        // its inialialization
        logSetting = new RimLogSetting(customization, configuration);
        // Initialize the adv setting controller
        advSettingsController.initialize();

        // Update the list of sources
        settingsController.updateListOfSources();
    }
    
    public void onDisplay() {
        super.onDisplay();
        computeLayout();
        invalidate();
    }
    
    public void onExpose() {
        super.onExposed();
        computeLayout();
        invalidate();
    }

    public void removeAllItems() {
        deleteAll();
    }

    private void computeLayout() {
        if (dirty) {
            Log.debug(TAG_LOG, "Recomputing layout");
            // Source settings are at the top of the screen
            for(int i=0;i<sourceSettingList.size();++i) {
                SettingsUISyncSource ss = (SettingsUISyncSource)sourceSettingList.elementAt(i);
                if (ss instanceof Field) {
                    Field f = (Field)ss;
                    add(f);
                } else {
                    Log.error(TAG_LOG, "Cannot display source settings because it is not a Field");
                }
            }
            // Now add the sync mode
            if (modeSetting != null) {
                final SeparatorField separator = new SeparatorField();
                add(separator);
                add(modeSetting);
            }
            // Add the log level
            if (logSetting != null) {
                final SeparatorField separator = new SeparatorField();
                add(separator);
                add(logSetting);
            }
            dirty = false;
        }
    }

    /**
     * Hide the entire log section. The section is removed by the settings
     * screen during the initialization depending by customization
     * implementation. No parameters are passed to this method for this reason
     */
    public void hideLogsSection() {
        logSetting = null;
        dirty = true;
    }

    public void hideResetSection() {
        // Reset is not here
    }

    /**
     * Hide the send log button. It can be requested by the customization
     * implementation.
     */
    public void hideSendLogCommand() {
        enableSendLog = false;
    }

    /**
     * Hide the view log button. It can be requested by the customization
     * implementation.
     */
    public void hideViewLogCommand() {
    }

    /**
     * Hide the import contacts section
     */
    public void hideImportContactsSection() {
    }

    /**
     * Disable the item related to the reset command. Useful not to have
     * concurrent sync or connections.
     * @param hidden true if hidden, false otherwise
     */
    public void enableResetCommand(boolean enable) {
    }

    /**
     * Disable the item related to the Send Log command. Useful not to have
     * concurrent sync or connections.
     * @param hidden true if hidden, false otherwise
     */
    public void enableSendLogCommand(boolean enable) {
        enableSendLog = enable;
    }

    /**
     * Accessor method to set the log level on the view selector item
     * @param logLevel the log level to be set
     */
    public void setViewLogLevel(int logLevel) {
        if (logSetting != null) {
            logSetting.setLogLevel(logLevel);
        }
    }

    /**
     * Accessor method to get the log level set on the view selector item
     * @param logLevel the log level to be set
     * @return int the log level set by the user on the view
     */
    public int getViewLogLevel() {
        if (logSetting != null) {
            return logSetting.getLogLevel();
        }
        return Log.INFO;
    }

    /**
     * Accessor method to enable/disable Bandwidth Saver
     * @param bandwidthSaverChecked true to enable, false to disable Bandwidth Saver
     */
    public void setBandwidthSaver(boolean enable) {
    }

     /**
     * Accessor method to get value fore Bandwith Saver status
     * @return boolean status enabled/disabled for Bandwidth Saver
     */
    public boolean getBandwidthSaver() {
        return false;
    }


     /**
     * Hide the entire Bandwidth Saver section. The section is removed by the settings
     * screen during the initialization depending by customization
     * implementation. No parameters are passed to this method for this reason
     */
    public void hideBandwidthSaverSection() {
    }

    /**
     * Set a SettingsUISyncSource object in the given position.
     * setSettingsUISyncSourceCount must be called before.
     * @param item
     * @param index
     */
    public void setSettingsUISyncSource(SettingsUISyncSource item, int index) {
        sourceSettingList.setElementAt(item, index);
        dirty = true;
    }

    /**
     * Specify the SettingsUISyncSource items count to be displayed in a
     * SyncSettingsScreen
     * @param count
     */
    public void setSettingsUISyncSourceCount(int count) {
        // Initialize the list of source settings
        sourceSettingList = new Vector();
        for(int i=0;i<count;++i) {
            sourceSettingList.addElement(null);
        }
        dirty = true;
    }

    /**
     * Add the SyncModeSetting
     * @return the new created SettingsUIItem
     */
    public SettingsUIItem addSyncModeSetting() {
        modeSetting = new RimSyncModeSetting(customization, configuration);
        dirty = true;
        return modeSetting;
    }

    public SettingsUIItem addSyncTypeSetting() {
        return null;
    }

    /**
     * Add the SyncIntervalSetting
     * @return the new created SettingsUIItem
     */
    public SettingsUIItem addSyncIntervalSetting() {
        return null;
    }

    /**
     * Add the C2SPushSetting
     * @return the new created SettingsUIItem
     */
    public SettingsUIItem addC2SPushSetting() {
        return null;
    }

    /**
     * Adds "Save" menu item to the menu on Configuration screen
     * 
     * @param Menu :
     *            menu to which save menuitem is to be added
     * @param int :
     *            instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance) {
        if (settingsController.hasChanges() ||
            (logSetting != null && logSetting.hasChanges()))
        {
            menu.add(saveMenu);
        }
        menu.add(backMenu);

        // We need a valid email configuration to send logs via email
        if (customization.enableFileLogging()) {
            if (customization.sendLogViaEmailEnabled() && BlackberryUtils.isBISEmailAvailable()) {
                // Since we had to re-implement the getContent method, we need
                // to make sure the log is in a file
                if (Log.getAppender() instanceof FileAppender) {
                    Session session = Session.getDefaultInstance();
                    if (session != null) {
                        menu.add(sendLogMenu);
                    }
                }
            }
        }
    }

    /**
     * Invoked on user hitting the escape button. It prompts the user with a
     * dialog, if he wants to save/discard/cancel the changes in configuration
     * 
     * @param void.
     * @return void.
     */
    public boolean onClose() {
        if (settingsController.hasChanges() ||
            (logSetting != null && logSetting.hasChanges())) {
            final int response = Dialog.ask(Dialog.D_SAVE, 
                    BBLocalization.getInstance().getLanguage("dialog_save"));
            if (Dialog.SAVE == response) {
                saveConfiguration();
            } else if (Dialog.DISCARD == response) {
                settingsController.updateListOfSources();
                advSettingsController.initialize();
            } else if (Dialog.CANCEL == response) {
                return false;
            }
        }
        controller.getScreenDisplayController().hideConfigScreen();
        return true;
    }


    /**
     * Invoked when a user selects the "Save" option from the menu in the
     * configuration screen or from the dialog when the user exits the
     * configuration screen. It saves changes on the configuration into the
     * persistant store
     * 
     */
    public boolean saveConfiguration() {
        Log.trace(TAG_LOG, "saveConfiguration");
        settingsController.saveSettings();
        advSettingsController.checkAndSave();
        return true;
    }

    public Object getUiScreen() {
        return this;
    }

    protected void onFocus(int arg0) {
        super.onFocus(arg0);
    }
}
