/*
 * Funambol is a mobile platform developed by Funambol, Inc. 
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission 
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE 
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 * 
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 * 
 * You should have received a copy of the GNU Affero General Public License 
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite 
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 * 
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 * 
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably 
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 *
 */

package com.funambol.blackberry.ui.rim;

import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.Font;
import net.rim.device.api.ui.MenuItem;
import net.rim.device.api.ui.Manager;
import net.rim.device.api.ui.component.LabelField;
import net.rim.device.api.ui.component.Menu;
import net.rim.device.api.ui.container.MainScreen;
import net.rim.device.api.ui.container.VerticalFieldManager;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.component.BitmapField;
import net.rim.device.api.ui.component.ButtonField;
import net.rim.device.api.ui.component.SeparatorField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.Color;
import net.rim.device.api.ui.container.FlowFieldManager;
import net.rim.device.api.ui.text.PhoneTextFilter;
import net.rim.device.api.math.Fixed32;
import net.rim.device.api.system.EncodedImage;
import net.rim.device.api.system.Display;
import net.rim.device.api.system.Bitmap;

import com.funambol.blackberry.BBAppSyncSourceManager;
import com.funambol.blackberry.BBConfiguration;
import com.funambol.blackberry.BBCustomization;
import com.funambol.blackberry.BBLocalization;
import com.funambol.blackberry.SyncClient;
import com.funambol.blackberry.controller.BBSignupScreenController;
import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.controller.UISyncSourceController;
import com.funambol.blackberry.source.BBAppSyncSource;
import com.funambol.blackberry.ui.UISyncSourceContainer;
import com.funambol.blackberry.ui.rim.component.ColoredLabelField;
import com.funambol.blackberry.ui.rim.component.ColoredSeparatorField;
import com.funambol.blackberry.ui.rim.component.ExtendedPasswordField;
import com.funambol.blackberry.ui.rim.component.JustifiedVerticalFieldManager;
import com.funambol.blackberry.ui.rim.component.LinkLabelField;
import com.funambol.blackberry.ui.rim.component.RoundedFieldManager;
import com.funambol.blackberry.ui.rim.component.SingleLineEditField;
import com.funambol.blackberry.ui.rim.component.SpacerField;
import com.funambol.client.controller.AccountScreenController;
import com.funambol.client.controller.SignupHandler;
import com.funambol.client.controller.SynchronizationController;
import com.funambol.client.ui.SignupScreen;
import com.funambol.client.ui.SynchronizationScreen;
import com.funambol.syncml.spds.SyncSource;
import com.funambol.util.Log;
import com.funambol.util.StringUtil;

import java.util.Enumeration;
import net.rim.device.api.ui.text.NumericTextFilter;


public class RimSignupScreen extends MainScreen implements SignupScreen, 
        UISyncSourceContainer, SynchronizationScreen {
    
    private static final String TAG_LOG = "RimSignupScreen";

    private SingleLineEditField       userEditField    = null;
    private ExtendedPasswordField     passEditField    = null;
    private String                    syncUrl          = null;

    private SingleLineEditField       capcthaEditField = null;
    private BitmapField               captchaImage     = null;
    
    private ButtonField               continueButton   = null;
    private ButtonField               signupButton     = null;

    private Field                     bodyCredentialField = null;
    private Field                     bodyCaptchaField    = null;

    private Manager                   bodyManager      = null;

    private Field                     firstScreenField = null;

    private boolean                   quitEnabled      = true;
    private boolean                   signupEnabled    = true;

    private BBLocalization            localization     = BBLocalization.getInstance();
    private BBCustomization           customization    = BBCustomization.getInstance();

    private BBSignupScreenController  screenController = null;
    private Controller                controller = null;

    private final MenuItem quitMenu   = new MenuItem(BBLocalization.getInstance()
                                               .getLanguage("menu_cancel"), 20000, 10) {
                                            public void run() {
                                                screenController.hide();
                                            }
                                        };

    private final MenuItem signupMenu = new MenuItem(BBLocalization.getInstance()
                                               .getLanguage("menu_signup"), 20000, 10) {
                                            public void run() {
                                                 screenController.signup();
                                            }
                                        };

    private final MenuItem continueMenu = new MenuItem(BBLocalization.getInstance()
                                               .getLanguage("menu_continue"), 20000, 10) {
                                            public void run() {
                                                 screenController.continueWithSignup();
                                            }
                                        };

    private Font bigFont, mediumFont;

    public RimSignupScreen(Controller ctrl) {
        super(Field.USE_ALL_WIDTH | Field.USE_ALL_HEIGHT |
              Field.FIELD_LEFT | Field.FIELD_TOP |
              Manager.VERTICAL_SCROLLBAR | Manager.VERTICAL_SCROLL);

        Log.debug(TAG_LOG, "Init Signup Screen");

        controller = ctrl;

        int validationMode = customization.getDefaultMSUValidationMode();

        // Init fonts
        Font defaultFont = RimHelper.getDefaultFont();
        bigFont = defaultFont;
        mediumFont = defaultFont.derive(defaultFont.getStyle(),
                (int)(defaultFont.getHeight()*0.8));
        if(isSmallWidthDevice()) {
            defaultFont = mediumFont;
        }
        this.setFont(defaultFont);

        // Init spacers size
        int smallSpacerHeight, mediumSpacerHeight, bigSpacerHeight;
        int defaultSpacerWidth, smallSpacerWidth;
        int bodyContentPadding;
        if(isSmallHeightDevice()) {
            smallSpacerHeight = 3;
            mediumSpacerHeight = 6;
            bigSpacerHeight = 10;
        } else {
            smallSpacerHeight = 5;
            mediumSpacerHeight = 9;
            bigSpacerHeight = 14;
        }
        if(isVerySmallWidthDevice()) {
            defaultSpacerWidth = 5;
            smallSpacerWidth = 2;
            bodyContentPadding = 3;
        } else if(isSmallWidthDevice()) {
            defaultSpacerWidth = 5;
            smallSpacerWidth = 5;
            bodyContentPadding = 5;
        } else {
            defaultSpacerWidth = 5;
            smallSpacerWidth = 5;
            bodyContentPadding = 10;
        }

        // Init title field
        LabelField title = new LabelField(customization.getApplicationTitle());
        title.setFont(getFont().derive(RimTheme.TITLE_FONT_STYLE,
                RimTheme.TITLE_FONT_SIZE));
        setTitle(title);

        // Init signup image  field
        EncodedImage topLogo;
        if(isSmallHeightDevice()) {
            topLogo = EncodedImage.getEncodedImageResource((String)customization
                .getSignupLogoSmall().getOpaqueDescriptor());
        } else {
            topLogo = EncodedImage.getEncodedImageResource((String)customization
                .getSignupLogo().getOpaqueDescriptor());
        }
        if(topLogo.getWidth() > Display.getWidth()) {
            topLogo = scaleImageToWidth(topLogo, Display.getWidth());
        }
        BitmapField topImageField = new BitmapField(topLogo.getBitmap(),
                Field.FIELD_HCENTER | Field.FIELD_TOP | Field.NON_FOCUSABLE);
        LabelField topCredField = new LabelField(
                localization.getLanguage("signup_help_text"), Field.FIELD_LEFT);
        topCredField.setFont(mediumFont);
        LabelField topCapcthaField = new LabelField(
                localization.getLanguage("signup_captcha_help_text"), Field.FIELD_LEFT);
        topCapcthaField.setFont(mediumFont);

        captchaImage = new BitmapField(null, Field.FIELD_HCENTER |
                Field.FIELD_TOP | Field.NON_FOCUSABLE);
        RoundedFieldManager captchaImageFieldContainer = new RoundedFieldManager(0);
        captchaImageFieldContainer.setFrameColor(Color.GRAY);
        captchaImageFieldContainer.setFrameArc(5);
        captchaImageFieldContainer.setFramePadding(5);
        captchaImageFieldContainer.setDrawShading(true);
        captchaImageFieldContainer.alignToField(captchaImage);
        captchaImageFieldContainer.add(captchaImage);
        
        capcthaEditField = new SingleLineEditField();
        capcthaEditField.setFilter(new NumericTextFilter());
        RoundedFieldManager captchaEditFieldContainer = new RoundedFieldManager(0);
        captchaEditFieldContainer.setFrameColor(RimTheme.DEFAULT_COLOR);
        captchaEditFieldContainer.setDrawShading(true);
        captchaEditFieldContainer.add(capcthaEditField);
        captchaEditFieldContainer.alignToField(captchaImage);

        VerticalFieldManager captchaFields = new VerticalFieldManager(Field.FIELD_HCENTER);
        captchaFields.add(captchaEditFieldContainer);
        captchaFields.add(SpacerField.createVerticalSpacer(mediumSpacerHeight));
        captchaFields.add(captchaImageFieldContainer);

        // Init username field
        LabelField usernameLabel = new LabelField(localization.getLanguage("signup_phone_number"));
        usernameLabel.setFont(defaultFont.derive(Font.BOLD));
        userEditField = new SingleLineEditField();
        userEditField.setFilter(new PhoneTextFilter());
        RoundedFieldManager userEditFieldContainer = new RoundedFieldManager(Field.USE_ALL_WIDTH);
        userEditFieldContainer.setFrameColor(RimTheme.DEFAULT_COLOR);
        userEditFieldContainer.setDrawShading(true);
        userEditFieldContainer.add(userEditField);

        // Init password field
        LabelField passwordLabel = new LabelField(localization.getLanguage("conf_password"));
        passwordLabel.setFont(defaultFont.derive(Font.BOLD));
        passEditField = new ExtendedPasswordField("",
                localization.getLanguage("conf_show_password"));

        int estimatedUserWidth = usernameLabel.getFont().getAdvance(usernameLabel.getText()) +
                usernameLabel.getFont().getAdvance("+") + 3;
        int estimatedPassWidth = passwordLabel.getFont().getAdvance(passwordLabel.getText());
        if(estimatedUserWidth > estimatedPassWidth) {
            estimatedPassWidth = estimatedUserWidth-estimatedPassWidth;
            estimatedUserWidth = 0;
        } else if(estimatedUserWidth < estimatedPassWidth) {
            estimatedUserWidth = estimatedPassWidth-estimatedUserWidth;
            estimatedPassWidth = 0;
        } else {
            estimatedUserWidth = 0;
            estimatedPassWidth = 0;
        }
        
        HorizontalFieldManager usernameField =
                new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        VerticalFieldManager usernameLabelContainer = new VerticalFieldManager();
        usernameLabelContainer.add(SpacerField.createVerticalSpacer(4));
        usernameLabelContainer.add(usernameLabel);
        VerticalFieldManager plusLabelContainer = new VerticalFieldManager();
        ColoredLabelField plusField = new ColoredLabelField("+");
        plusField.setFont(defaultFont.derive(Font.BOLD));
        plusField.setColor(Color.GRAY);
        plusLabelContainer.add(SpacerField.createVerticalSpacer(3));
        plusLabelContainer.add(plusField);
        usernameField.add(usernameLabelContainer);
        usernameField.add(SpacerField.createHorizontalSpacer(
                defaultSpacerWidth + estimatedUserWidth));
        usernameField.add(plusLabelContainer);
        usernameField.add(SpacerField.createHorizontalSpacer(2));
        usernameField.add(userEditFieldContainer);

        HorizontalFieldManager passwordField =
                new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        VerticalFieldManager passwordLabelContainer = new VerticalFieldManager();
        passwordLabelContainer.add(SpacerField.createVerticalSpacer(4));
        passwordLabelContainer.add(passwordLabel);
        passwordField.add(passwordLabelContainer);
        passwordField.add(SpacerField.createHorizontalSpacer(
                defaultSpacerWidth + estimatedPassWidth));
        passwordField.add(passEditField);

        VerticalFieldManager credentialsField = new VerticalFieldManager(Field.USE_ALL_WIDTH);
        credentialsField.add(usernameField);
        credentialsField.add(SpacerField.createVerticalSpacer(bigSpacerHeight));
        credentialsField.add(passwordField);

        // Init terms and conditions field
        String termsLabel1 = localization.getLanguage("signup_terms_and_conditions1");
        String termsLabel2 = localization.getLanguage("signup_terms_and_conditions3");
        String[] termsLabels1 = StringUtil.split(termsLabel1, " ");
        String[] termsLabels2 = StringUtil.split(termsLabel2, " ");

        LinkLabelField termsLink1 = new LinkLabelField(
                localization.getLanguage("signup_terms_and_conditions2"),
                RimTheme.DEFAULT_COLOR, Field.FIELD_HCENTER);
        termsLink1.setWebLink(customization.getTermsAndConditionsUrl());
        termsLink1.setFont(mediumFont);
        ColoredLabelField termsLabelField2 = new ColoredLabelField(" " +
                localization.getLanguage("signup_terms_and_conditions3") + " ",
                Field.FIELD_HCENTER);
        termsLabelField2.setFont(mediumFont);
        LinkLabelField termsLink2 = new LinkLabelField(
                localization.getLanguage("signup_terms_and_conditions4"),
                RimTheme.DEFAULT_COLOR, Field.FIELD_HCENTER);
        termsLink2.setWebLink(customization.getPrivacyPolicyUrl());
        termsLink2.setFont(mediumFont);
        
        FlowFieldManager termsField = new FlowFieldManager(Field.FIELD_HCENTER);
        for(int i=0; i<termsLabels1.length; i++) {
            StringBuffer label = new StringBuffer(termsLabels1[i]);
            label.append(" ");
            ColoredLabelField termsLabelField = new ColoredLabelField(label);
            termsLabelField.setFont(mediumFont);
            termsField.add(termsLabelField);
        }
        termsField.add(termsLink1);
        for(int i=0; i<termsLabels2.length; i++) {
            StringBuffer label = new StringBuffer(" ");
            label.append(termsLabels2[i]);
            // Add space to the last word
            if(i == termsLabels2.length-1) {
                label.append(" ");
            }
            ColoredLabelField termsLabelField = new ColoredLabelField(label);
            termsLabelField.setFont(mediumFont);
            termsField.add(termsLabelField);
        }
        termsField.add(termsLink2);

        // Init continue button
        continueButton = new ButtonField(localization.getLanguage("signup_continue_button"),
                Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        continueButton.setFont(bigFont);
        continueButton.setChangeListener(new FieldChangeListener() {
            public void fieldChanged(Field field, int i) {
                screenController.continueWithSignup();
            }
        });

        // Init signup button
        signupButton = new ButtonField(localization.getLanguage("signup_button"),
                Field.FIELD_HCENTER | ButtonField.CONSUME_CLICK);
        signupButton.setFont(bigFont);
        signupButton.setChangeListener(new FieldChangeListener() {
            public void fieldChanged(Field field, int i) {
                screenController.signup();
            }
        });

        VerticalFieldManager headerManager = new VerticalFieldManager(Field.USE_ALL_WIDTH |
                Field.FIELD_HCENTER);
        int headerVerticalSpacer = isSmallHeightDevice() ?
            mediumSpacerHeight : smallSpacerHeight;
        firstScreenField = SpacerField.createVerticalSpacer(headerVerticalSpacer, Field.FOCUSABLE);
        headerManager.add(firstScreenField);

        headerManager.add(topImageField);
        if(isSmallHeightDevice()) {
            headerManager.add(SpacerField.createVerticalSpacer(mediumSpacerHeight));
        } else {
            headerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));
        }
        headerManager.add(createSeparatorField());
        
        ExtendedVerticalFieldManager bodyCredFields = new ExtendedVerticalFieldManager(
                bodyContentPadding, Field.USE_ALL_WIDTH);
        bodyCredFields.add(SpacerField.createVerticalSpacer(bodyContentPadding));
        bodyCredFields.add(topCredField);
        bodyCredFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + smallSpacerHeight));
        bodyCredFields.add(credentialsField);
        bodyCredFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + mediumSpacerHeight));
        if(validationMode == SignupHandler.VALIDATION_MODE_CAPTCHA) {
            bodyCredFields.add(continueButton);
        } else {
            bodyCredFields.add(termsField);
            bodyCredFields.add(SpacerField.createVerticalSpacer(mediumSpacerHeight));
            bodyCredFields.add(signupButton);
        }
        bodyCredFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + mediumSpacerHeight));
        bodyCredentialField = bodyCredFields;

        if(validationMode == SignupHandler.VALIDATION_MODE_CAPTCHA) {
            ExtendedVerticalFieldManager bodyCaptchaFields = new ExtendedVerticalFieldManager(
                    bodyContentPadding, Field.USE_ALL_WIDTH);
            bodyCaptchaFields.add(SpacerField.createVerticalSpacer(bodyContentPadding));
            bodyCaptchaFields.add(topCapcthaField);
            bodyCaptchaFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + smallSpacerHeight));
            bodyCaptchaFields.add(captchaFields);
            bodyCaptchaFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + mediumSpacerHeight));
            bodyCaptchaFields.add(termsField);
            bodyCaptchaFields.add(SpacerField.createVerticalSpacer(mediumSpacerHeight));
            bodyCaptchaFields.add(signupButton);
            bodyCaptchaFields.add(SpacerField.createVerticalSpacer(bigSpacerHeight + mediumSpacerHeight));
            bodyCaptchaField = bodyCaptchaFields;
        }

        bodyManager = new HorizontalFieldManager(Field.USE_ALL_WIDTH);
        bodyManager.add(SpacerField.createHorizontalSpacer(bodyContentPadding));

        VerticalFieldManager headerBodyManager = new VerticalFieldManager(Field.FIELD_TOP);
        headerBodyManager.add(headerManager);
        headerBodyManager.add(bodyManager);

        // Init the footer fields
        HorizontalFieldManager footerContent = new HorizontalFieldManager(Field.FIELD_RIGHT);
        LabelField accountLabel = new LabelField(
                localization.getLanguage("signup_login_label"),
                Field.FIELD_VCENTER);
        if(isVerySmallWidthDevice()) {
            accountLabel.setFont(mediumFont);
        } else {
            accountLabel.setFont(defaultFont.derive(Font.BOLD));
        }
        ButtonField loginButton = new ButtonField(
                localization.getLanguage("signup_login_button"),
                ButtonField.CONSUME_CLICK);
        loginButton.setFont(mediumFont);
        loginButton.setChangeListener(new FieldChangeListener() {
            public void fieldChanged(Field field, int i) {
                screenController.switchToLoginScreen();
            }
        });
        footerContent.add(accountLabel);
        footerContent.add(SpacerField.createHorizontalSpacer(smallSpacerWidth));
        footerContent.add(loginButton);
        footerContent.add(SpacerField.createHorizontalSpacer(smallSpacerWidth));

        VerticalFieldManager footerManager = new VerticalFieldManager();
        footerManager.add(createSeparatorField());
        footerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));
        footerManager.add(footerContent);
        footerManager.add(SpacerField.createVerticalSpacer(smallSpacerHeight));

        JustifiedVerticalFieldManager justifiedManager = 
                new JustifiedVerticalFieldManager(headerBodyManager, footerManager,
                true, Field.USE_ALL_HEIGHT | Field.USE_ALL_WIDTH,
                Display.getHeight() - /* Expected title height*/ 22);

        add(justifiedManager);

        updateFocusedField();

        screenController = new BBSignupScreenController(controller,
                BBCustomization.getInstance(),
                BBConfiguration.getInstance(),
                BBLocalization.getInstance(),
                BBAppSyncSourceManager.getInstance(),
                this);

        controller.setSignupScreenController(screenController);

        screenController.initScreen();
        initialize(screenController);
    }

    public void initialize(AccountScreenController c) {
        BBAppSyncSource appSource = (BBAppSyncSource)
                BBAppSyncSourceManager.getInstance().getSource(
                BBAppSyncSourceManager.SYNC_FLAG_CONFIG);
        UISyncSourceController itemController = new UISyncSourceController(
                                                 controller, appSource);
        SyncSource ss = appSource.getSyncSource();
        ss.setListener(itemController);
        appSource.setClientUISyncSourceController(itemController);
    }

    private boolean isSmallHeightDevice() {
        return (Display.getHeight() <= 320);
    }

    private boolean isSmallWidthDevice() {
        return (Display.getWidth() <= 320);
    }

    private boolean isVerySmallWidthDevice() {
        return (Display.getWidth() <= 240);
    }

    private static EncodedImage scaleImageToWidth(EncodedImage encoded, int newWidth) {
        return scaleToFactor(encoded, encoded.getWidth(), newWidth);
    }

    private static EncodedImage scaleToFactor(EncodedImage encoded, int curSize, int newSize) {
        int numerator = Fixed32.toFP(curSize);
        int denominator = Fixed32.toFP(newSize);
        int scale = Fixed32.div(numerator, denominator);

        return encoded.scaleImage32(scale, scale);
    }

    private SeparatorField createSeparatorField() {
        ColoredSeparatorField separator = new ColoredSeparatorField(0);
        separator.setColor(RimTheme.DEFAULT_COLOR);
        separator.setDrawShading(true);
        return separator;
    }

    public void promptCredentials() {
        synchronized (SyncClient.getEventLock()) {
            if(bodyManager.getFieldCount() == 2) {
                if(bodyManager.getField(1) == bodyCaptchaField) {
                    bodyManager.delete(bodyCaptchaField);
                    bodyManager.add(bodyCredentialField);
                }
            } else if(bodyManager.getFieldCount() == 1 &&
                    bodyCredentialField != null) {
                bodyManager.add(bodyCredentialField);
            }
            updateFocusedField();
        }
    }

    private boolean isCredentialsPrompted() {
        if(bodyManager.getFieldCount() == 2) {
            return bodyManager.getField(1) == bodyCredentialField;
        }
        return false;
    }

    public void promptCaptcha() {
        synchronized (SyncClient.getEventLock()) {
            if(bodyManager.getFieldCount() == 2) {
                if(bodyManager.getField(1) == bodyCredentialField) {
                    bodyManager.delete(bodyCredentialField);
                    bodyManager.add(bodyCaptchaField);
                }
            } else if(bodyManager.getFieldCount() == 1 &&
                    bodyCaptchaField != null) {
                bodyManager.add(bodyCaptchaField);
            }
            updateFocusedField();
        }
    }

    private boolean isCaptchaPrompted() {
        if(bodyManager.getFieldCount() == 2) {
            return bodyManager.getField(1) == bodyCaptchaField;
        }
        return false;
    }

    public void setCaptchaImage(byte[] image) {
        Enumeration m = EncodedImage.getSupportedMIMETypes();
        captchaImage.setBitmap(Bitmap.createBitmapFromBytes(image, 0,
                image.length, 1 /* 1 means no scaling */));
    }

    public String getCaptchaToken() {
        return capcthaEditField.getText();
    }

    public void setCaptchaToken(String token) {
        capcthaEditField.setText(token);
    }

    public void addShowPasswordField(boolean checked) {
        passEditField.addShowPasswordField(checked, mediumFont);
    }

    public void disableSignup() {
        signupEnabled = false;
    }

    public void enableSignup() {
        signupEnabled = true;
    }

    public void disableSave() { }

    public void enableSave() { }

    public void enableCancel() {
        quitEnabled = true;
    }

    public void disableCancel() {
        quitEnabled = false;
    }

    public void checkFailed() { }

    public void checkSucceeded() { }

    public SynchronizationController getSynchronizationController() {
        return screenController;
    }

    public void redraw() {
        invalidate();
    }

    /**
     * Invoked on user hitting the escape button. 
     */
    public boolean onClose() {
        screenController.hide();
        return true;
    }

    public Object getUiScreen() {
        return this;
    }

    /**
     * Adds "Save" menu item to the menu on Configuration screen
     *
     * @param Menu
     *            : menu to which save menuitem is to be added
     * @param int : instance
     * @return void.
     */
    public void makeMenu(Menu menu, int instance) {
        if(signupEnabled) {
            int validationMode = customization.getDefaultMSUValidationMode();
            if(isCredentialsPrompted() &&
                    validationMode == SignupHandler.VALIDATION_MODE_CAPTCHA) {
                menu.add(continueMenu);
            } else {
                menu.add(signupMenu);
            }
        }
        if(quitEnabled) {
            menu.add(quitMenu);
        }
    }

    public void setSyncUrl(String url) {
        syncUrl = url;
    }

    public String getSyncUrl() {
        return syncUrl;
    }

    public void setUsername(String username) {
        userEditField.setText(filterPhoneNumber(username));
    }

    public String getUsername() {
        return filterPhoneNumber(userEditField.getText());
    }

    public void setPassword(String password) {
        passEditField.setText(password);
    }

    public String getPassword() {
        return passEditField.getText();
    }

    public boolean isPasswordShowed() {
        return passEditField.isPasswordShowed();
    }

    private String filterPhoneNumber(String phone) {
        StringBuffer filtered = new StringBuffer(phone.length());
        for(int i=0; i<phone.length(); i++) {
            char c = phone.charAt(i);
            if(isValidphoneNumberChar(c)) {
                filtered.append(c);
            }
        }
        return filtered.toString().trim();
    }

    private boolean isValidphoneNumberChar(char c) {
        final char[] VALID_INPUT = {'0','1','2','3','4','5','6','7','8','9',
                                    '(',')','+','-',' '};
        for(int i=0; i<VALID_INPUT.length; i++) {
            if(c == VALID_INPUT[i]) {
                return true;
            }
        }
        return false;
    }

    protected void onExposed() {
        super.onExposed();
        if(screenController instanceof BBSignupScreenController) {
            ((BBSignupScreenController)screenController)
                    .showNetworkUsageWarningIfRequired();
        }
    }

    protected void onDisplay() {
        super.onDisplay();
        if(screenController instanceof BBSignupScreenController) {
            ((BBSignupScreenController)screenController)
                    .showNetworkUsageWarningIfRequired();
        }
    }

    private void updateFocusedField() {
        // Give the focus to the first field
        firstScreenField.setFocus();
    }

    private class ExtendedVerticalFieldManager extends VerticalFieldManager {

        private int paddingRight;
        
        public ExtendedVerticalFieldManager(int paddingRight, long style) {
            super(style);
            this.paddingRight = paddingRight;
        }

        protected void sublayout(int maxWidth, int maxHeight) {
            super.sublayout(maxWidth-paddingRight, maxHeight);
        }
    }
}
