/*
 * Funambol is a mobile platform developed by Funambol, Inc.
 * Copyright (C) 2010 Funambol, Inc.
 * 
 * This program is free software; you can redistribute it and/or modify it under
 * the terms of the GNU Affero General Public License version 3 as published by
 * the Free Software Foundation with the addition of the following permission
 * added to Section 15 as permitted in Section 7(a): FOR ANY PART OF THE COVERED
 * WORK IN WHICH THE COPYRIGHT IS OWNED BY FUNAMBOL, FUNAMBOL DISCLAIMS THE
 * WARRANTY OF NON INFRINGEMENT  OF THIRD PARTY RIGHTS.
 *
 * This program is distributed in the hope that it will be useful, but WITHOUT
 * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
 * FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
 * details.
 *
 * You should have received a copy of the GNU Affero General Public License
 * along with this program; if not, see http://www.gnu.org/licenses or write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301 USA.
 * 
 * You can contact Funambol, Inc. headquarters at 643 Bair Island Road, Suite
 * 305, Redwood City, CA 94063, USA, or at email address info@funambol.com.
 *
 * The interactive user interfaces in modified source and object code versions
 * of this program must display Appropriate Legal Notices, as required under
 * Section 5 of the GNU Affero General Public License version 3.
 *
 * In accordance with Section 7(b) of the GNU Affero General Public License
 * version 3, these Appropriate Legal Notices must retain the display of the
 * "Powered by Funambol" logo. If the display of the logo is not reasonably
 * feasible for technical reasons, the Appropriate Legal Notices must display
 * the words "Powered by Funambol".
 */

package com.funambol.blackberry.ui.rim;

import com.funambol.blackberry.BBLocalization;
import net.rim.device.api.ui.Field;
import net.rim.device.api.ui.FieldChangeListener;
import net.rim.device.api.ui.Font;

import net.rim.device.api.ui.component.CheckboxField;
import net.rim.device.api.ui.container.HorizontalFieldManager;
import net.rim.device.api.ui.container.VerticalFieldManager;

import com.funambol.blackberry.controller.Controller;
import com.funambol.blackberry.ui.rim.component.ColoredLabelField;

import com.funambol.client.source.AppSyncSource;
import com.funambol.client.source.AppSyncSourceConfig;
import com.funambol.client.ui.SettingsUISyncSource;
import com.funambol.client.ui.Bitmap;
import com.funambol.client.configuration.Configuration;
import com.funambol.util.BlackberryUtils;
import com.funambol.util.Log;
import net.rim.device.api.ui.component.BasicEditField;


/**
 * This class creates the configuration screen for configuring parameters
 * required for the SyncClient. In particular it supports the advanced settings
 * and the sync settings.
 */
public class RimSourceBasicSettings extends VerticalFieldManager implements SettingsUISyncSource {

    private static final String TAG_LOG = "RimSourceBasicSettings";

    protected boolean dirty;
    protected AppSyncSource source;
    protected BasicEditField uriField;
    protected ColoredLabelField enableLabel;
    protected CheckboxField enableField;

    protected HorizontalFieldManager hfm;
    protected VerticalFieldManager   vfm;
    protected ChangeListener         cl;

    protected Controller controller = null;

    protected BBLocalization localization = BBLocalization.getInstance();

    public RimSourceBasicSettings() {
        // TODO use the right style
        super();

        enableLabel = new ColoredLabelField();
        enableLabel.setFont(getFont().derive(Font.BOLD));

        enableField = getCheckboxField("", false);
        uriField = new BasicEditField(localization.getLanguage("conf_remote-uri"), "");

        // Register a change listener
        cl = new ChangeListener();
        uriField.setChangeListener(cl);
        enableField.setChangeListener(cl);
    }

    private CheckboxField getCheckboxField(String label, boolean checked) {
        // On BB OS 6.0 the standared CheckboxField fills the entire screen width
        // so we should use a custom one with fixed width
        if(BlackberryUtils.checkOS(BlackberryUtils.getOSVersion(), 6, 0, 0)) {
            return new FixedWidthCheckboxField(label, checked);
        } else {
            return new CheckboxField(label, checked);
        }
    }

    public void setRemoteUri(String remoteUri) {
        uriField.setText(remoteUri);
    }

    public String getRemoteUri() {
        if (uriField != null) {
            return uriField.getText();
        }
        return null;
    }

    public void setTitle(String value) {
        enableLabel.setText(value);
    }

    public void setAvailableSyncModes(int[] modes) {
        // Ignore this at the moment
    }
    
    public void setSyncMode(int mode) {
        // Ignore it
    }

    public int getSyncMode()  {
        throw new IllegalArgumentException("This method shall never be invoked");
    }

    public void setEnabledIcon(Bitmap icon) {
        // We ignore the icon in this implementation
    }

    public void setDisabledIcon(Bitmap icon) {
        // We ignore the icon in this implementation
    }

    public void setEnabled(boolean flag) {
        enableField.setChecked(flag);
    }

    public boolean getEnabled() {
        return enableField.getChecked();
    }

    public AppSyncSource getSource() {
        return source;
    }

    public void setSource(AppSyncSource source) {
        this.source = source;
    }

    public void setController(Controller controller) {
        this.controller = controller;
    }

    public void layout() {

        hfm = new HorizontalFieldManager();
        vfm = new VerticalFieldManager();

        hfm.add(enableField);

        if (enableLabel != null) {
            vfm.add(enableLabel);
        }

        if (uriField != null) {
            vfm.add(uriField);
        }

        hfm.add(vfm);

        this.add(hfm);
    }

    public boolean hasChanges() {
        return dirty;
    }

    public void saveSettings(Configuration configuration) {
        if (source != null) {
            Log.trace(TAG_LOG, "Saving settings for " + source.getName());
            AppSyncSourceConfig appConfig = source.getConfig();
            String uri = getRemoteUri();
            appConfig.setUri(uri);
            boolean enabled = getEnabled();
            appConfig.setEnabled(enabled);
        }
        dirty = false;
    }

    public void loadSettings(Configuration configuration) {
        if (source != null) {
            Log.trace(TAG_LOG, "Loading settings for " + source.getName());
            AppSyncSourceConfig appConfig = source.getConfig();
            String uri = appConfig.getUri();
            setRemoteUri(uri);
            boolean enabled = appConfig.getEnabled();
            setEnabled(enabled);
        }
        dirty = false;
    }

    public void saveCustomSettings() {
    }

    public void loadCustomSettings() {
    }

    protected class ChangeListener implements FieldChangeListener {

        public void fieldChanged(Field field, int context) {
            if (context != FieldChangeListener.PROGRAMMATIC) {
                dirty = true;
            }
        }
    }

    /**
     * Defines a CheckboxField with a fixed width
     */
    private class FixedWidthCheckboxField extends CheckboxField {

        public FixedWidthCheckboxField(String label, boolean checked) {
            super(label, checked);
        }

        protected void layout(int width, int height) {
            super.layout(24 /* fixed checkbox width + padding */, height);
        }
    }
}

 
